"""
Test that template instaniations of std::vector<long> and <short> in the same module have the correct types.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class UniqueTypesTestCase(TestBase):
    def test(self):
        """Test for unique types of std::vector<long> and std::vector<short>."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// Set breakpoint here", lldb.SBFileSpec("main.cpp")
        )

        # Do a "frame variable --show-types longs" and verify "long" is in each
        # line of output.
        self.runCmd("frame variable --show-types longs")
        output = self.res.GetOutput()
        for x in [line.strip() for line in output.split(os.linesep)]:
            # Skip empty line, closing brace, and messages about more variables
            # than can be displayed.
            if (
                not x
                or x == "}"
                or x == "..."
                or "Some of your variables have more members than the debugger will show by default"
                in x
            ):
                continue
            self.expect(x, "Expect type 'long'", exe=False, substrs=["long"])

        # Do a "frame variable --show-types shorts" and verify "short" is in
        # each line of output.
        self.runCmd("frame variable --show-types shorts")
        output = self.res.GetOutput()
        for x in [line.strip() for line in output.split(os.linesep)]:
            # Skip empty line, closing brace, and messages about more variables
            # than can be displayed.
            if (
                not x
                or x == "}"
                or x == "..."
                or "Some of your variables have more members than the debugger will show by default"
                in x
            ):
                continue
            self.expect(x, "Expect type 'short'", exe=False, substrs=["short"])



"""
Test that we can call C++ template fucntions.
"""
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TemplateFunctionsTestCase(TestBase):
    def do_test_template_function(self, add_cast):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )

        if add_cast:
            self.expect_expr("(int) foo(42)", result_type="int", result_value="42")
        else:
            self.expect(
                "expr b1 <=> b2",
                error=True,
                substrs=[
                    "warning: <user expression 0>:1:4: '<=>' is a single token in C++20; add a space to avoid a change in behavior"
                ],
            )

            self.expect_expr("foo(42)", result_type="int", result_value="42")

            # overload with template case
            self.expect_expr("h(10)", result_type="int", result_value="10")

            # ADL lookup case
            self.expect_expr("f(A::C{})", result_type="int", result_value="4")

            # ADL lookup but no overload
            self.expect_expr("g(A::C{})", result_type="int", result_value="4")

            # variadic function cases
            self.expect_expr("var(1)", result_type="int", result_value="10")
            self.expect_expr("var(1, 2)", result_type="int", result_value="10")

            # Overloaded templated operator case
            self.expect_expr("b1 > b2", result_type="bool", result_value="true")
            self.expect_expr("b1 >> b2", result_type="bool", result_value="true")
            self.expect_expr("b1 << b2", result_type="bool", result_value="true")
            self.expect_expr("b1 == b2", result_type="bool", result_value="true")

            # Overloaded operator case
            self.expect_expr("d1 > d2", result_type="bool", result_value="true")
            self.expect_expr("d1 >> d2", result_type="bool", result_value="true")
            self.expect_expr("d1 << d2", result_type="bool", result_value="true")
            self.expect_expr("d1 == d2", result_type="bool", result_value="true")

    @skipIfWindows
    def test_template_function_with_cast(self):
        self.do_test_template_function(True)

    @skipIfWindows
    @expectedFailureAll(debug_info=["dwarf", "gmodules", "dwo"])
    def test_template_function_without_cast(self):
        self.do_test_template_function(False)



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil


class TestCase(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    # Check that the CFA chain is correctly built
    def check_cfas(self, async_frames, process):
        async_cfas = list(map(lambda frame: frame.GetCFA(), async_frames))
        expected_cfas = [async_cfas[0]]
        # The CFA chain ends in nullptr.
        while expected_cfas[-1] != 0:
            error = lldb.SBError()
            expected_cfas.append(
                process.ReadPointerFromMemory(expected_cfas[-1], error)
            )
            self.assertSuccess(error, "Managed to read cfa memory")

        self.assertEqual(async_cfas, expected_cfas[:-1])

    def check_pcs(self, async_frames, process, target):
        for idx, frame in enumerate(async_frames[:-1]):
            # Read the continuation pointer from the second field of the CFA.
            error = lldb.SBError()
            continuation_ptr = process.ReadPointerFromMemory(
                frame.GetCFA() + target.addr_size, error
            )
            self.assertSuccess(error, "Managed to read context memory")

            # The PC of the previous frame should be the continuation pointer
            # with the funclet's prologue skipped.
            parent_frame = async_frames[idx + 1]
            prologue_to_skip = parent_frame.GetFunction().GetPrologueByteSize()
            self.assertEqual(continuation_ptr + prologue_to_skip, parent_frame.GetPC())


    def check_variables(self, async_frames, expected_values):
        for (frame, expected_value) in zip(async_frames, expected_values):
            myvar = frame.FindVariable("myvar")
            lldbutil.check_variable(self, myvar, False, value=expected_value)

    @swiftTest
    @skipIf(oslist=["windows", "linux"])
    def test(self):
        """Test `frame variable` in async functions"""
        self.build()

        source_file = lldb.SBFileSpec("main.swift")
        target, process, _, _ = lldbutil.run_to_source_breakpoint(
            self, "breakpoint1", source_file
        )

        async_frames = process.GetSelectedThread().frames
        self.check_cfas(async_frames, process)
        self.check_pcs(async_frames, process, target)
        self.check_variables(async_frames, ["222", "333", "444", "555"])

        target.DeleteAllBreakpoints()
        target.BreakpointCreateBySourceRegex("breakpoint2", source_file)
        process.Continue()
        # First frame is from a synchronous function
        frames = process.GetSelectedThread().frames
        async_frames = frames[1:]
        self.check_cfas(async_frames, process)
        self.check_pcs(async_frames, process, target)
        self.check_variables(async_frames, ["111", "222", "333", "444", "555"])

        target.DeleteAllBreakpoints()
        target.BreakpointCreateBySourceRegex("breakpoint3", source_file)
        process.Continue()
        async_frames = process.GetSelectedThread().frames
        self.check_cfas(async_frames, process)
        self.check_pcs(async_frames, process, target)
        self.check_variables(async_frames, ["222", "333", "444", "555"])



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestArm64eAttach(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    # On Darwin systems, arch arm64e means ARMv8.3 with ptrauth ABI used.
    @skipIf(archs=no_match(["arm64e"]))
    def test(self):
        # Skip this test if not running on AArch64 target that supports PAC
        if not self.isAArch64PAuth():
            self.skipTest("Target must support pointer authentication.")

        self.build()
        popen = self.spawnSubprocess(self.getBuildArtifact(), [])

        # This simulates how Xcode attaches to a process by pid/name.
        error = lldb.SBError()
        target = self.dbg.CreateTarget("", "arm64", "", True, error)
        listener = lldb.SBListener("my.attach.listener")
        process = target.AttachToProcessWithID(listener, popen.pid, error)
        self.assertSuccess(error)
        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertEqual(target.GetTriple().split('-')[0], "arm64e",
                         "target triple is updated correctly")

        self.expect('process plugin packet send qProcessInfo',
                    "debugserver returns correct triple",
                    substrs=['cputype:100000c', 'cpusubtype:2', 'ptrsize:8'])

        error = process.Kill()
        self.assertSuccess(error)



# TestSwiftHashedContainerEnum.py

"""
Test combinations of hashed swift containers with enums as keys/values
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2

class TestSwiftHashedContainerEnum(TestBase):
    @swiftTest
    def test_any_object_type(self):
        """Test combinations of hashed swift containers with enums"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, '// break here', lldb.SBFileSpec('main.swift'))

        self.expect(
            'frame variable -d run -- testA',
            ordered=False,
            substrs=[
                'key = c',
                'value = 1',
                'key = b',
                'value = 2'])
        self.expect(
            'expr -d run -- testA',
            ordered=False,
            substrs=[
                'key = c',
                'value = 1',
                'key = b',
                'value = 2'])

        self.expect(
            'frame variable -d run -- testB',
            ordered=False,
            substrs=[
                'key = "a", value = 1',
                'key = "b", value = 2'])
        self.expect(
            'expr -d run -- testB',
            ordered=False,
            substrs=[
                'key = "a", value = 1',
                'key = "b", value = 2'])

        self.expect(
            'frame variable -d run -- testC',
            ordered=False,
            substrs=['key = b', 'value = 2'])
        self.expect(
            'expr -d run -- testC',
            ordered=False,
            substrs=['key = b', 'value = 2'])

        self.expect(
            'frame variable -d run -- testD',
            substrs=['[0] = c'])
        self.expect(
            'expr -d run -- testD',
            substrs=['[0] = c'])



"""
Watch 4 bytes which spawn two doubleword aligned regions.
On a target that supports 8 byte watchpoints, this will
need to be implemented with a hardware watchpoint on both
doublewords.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class UnalignedWatchpointTestCase(TestBase):
    def hit_watchpoint_and_continue(self, process, iter_str):
        process.Continue()
        self.assertEqual(process.GetState(), lldb.eStateStopped, iter_str)
        thread = process.GetSelectedThread()
        self.assertEqual(thread.GetStopReason(), lldb.eStopReasonWatchpoint, iter_str)
        self.assertEqual(thread.GetStopReasonDataCount(), 1, iter_str)
        wp_num = thread.GetStopReasonDataAtIndex(0)
        self.assertEqual(wp_num, 1, iter_str)

    NO_DEBUG_INFO_TESTCASE = True

    # debugserver on AArch64 has this feature.
    @skipIf(archs=no_match(["arm64", "arm64e", "aarch64"]))
    @skipUnlessDarwin
    # debugserver only started returning an exception address within
    # a range lldb expects in https://reviews.llvm.org/D147820 2023-04-12.
    # older debugservers will return the base address of the doubleword
    # which lldb doesn't understand, and will stop executing without a
    # proper stop reason.
    @skipIfOutOfTreeDebugserver
    def test_unaligned_watchpoint(self):
        """Test a watchpoint that is handled by two hardware watchpoint registers."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "break here", self.main_source_file
        )

        thread.StepOver()

        frame = thread.GetFrameAtIndex(0)

        a_bytebuf_6 = frame.GetValueForVariablePath("a.bytebuf[6]")
        a_bytebuf_6_addr = a_bytebuf_6.GetAddress().GetLoadAddress(target)
        err = lldb.SBError()
        wp_opts = lldb.SBWatchpointOptions()
        wp_opts.SetWatchpointTypeWrite(lldb.eWatchpointWriteTypeOnModify)
        wp = target.WatchpointCreateByAddress(a_bytebuf_6_addr, 4, wp_opts, err)
        self.assertTrue(err.Success())
        self.assertTrue(wp.IsEnabled())
        self.assertEqual(wp.GetWatchSize(), 4)
        self.assertGreater(
            wp.GetWatchAddress() % 8, 4, "watched region spans two doublewords"
        )

        # We will hit our watchpoint 6 times during the execution
        # of the inferior.  If the remote stub does not actually split
        # the watched region into two doubleword watchpoints, we will
        # exit before we get to 6 watchpoint hits.
        for i in range(1, 7):
            self.hit_watchpoint_and_continue(process, "wp hit number %s" % i)



"""
Test that "breakpoint set -a" uses the ABI plugin to remove non-address bits
before attempting to set a breakpoint.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class AArch64LinuxNonAddressBitCodeBreak(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def do_tagged_break(self, hardware):
        if not self.isAArch64PAuth():
            self.skipTest("Target must support pointer authentication.")

        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.c",
            line_number("main.c", "// Set break point at this line."),
            num_expected_locations=1,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        if self.process().GetState() == lldb.eStateExited:
            self.fail("Test program failed to run.")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        cmd = "breakpoint set -a fnptr"
        # LLDB only has the option to force hardware break, not software.
        # It prefers sofware break when it can and this will be one of those cases.
        if hardware:
            cmd += " --hardware"
        self.expect(cmd)

        self.runCmd("continue")
        self.assertEqual(self.process().GetState(), lldb.eStateStopped)
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "`foo at main.c", "stop reason = breakpoint"],
        )

    # AArch64 Linux always enables the top byte ignore feature
    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_software_break(self):
        self.do_tagged_break(False)

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_hardware_break(self):
        self.do_tagged_break(True)



"""
Tests lldbutil's behavior when running to a source breakpoint fails.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class LLDBUtilFailedToHitBreakpointTest(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @expectedFailureAll(oslist=["windows"])
    def test_error_message(self):
        """
        Tests that run_to_source_breakpoint prints the right error message
        when failing to hit the wanted breakpoint.
        """
        self.build()
        with self.assertRaisesRegex(
            AssertionError,
            "Test process is not stopped at breakpoint: state: exited, exit code: 0, stdout: 'stdout_needlestderr_needle'",
        ):
            lldbutil.run_to_source_breakpoint(
                self, "// break here", lldb.SBFileSpec("main.cpp")
            )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2

class TestSwiftClangImporterExplicitCC1(TestBase):

    NO_DEBUG_INFO_TESTCASE = True

    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(setting=('symbols.swift-precise-compiler-invocation', 'false'))
    @skipIf(setting=('plugin.typesystem.clang.experimental-redecl-completion', 'true'), bugnumber='rdar://128094135')
    @skipUnlessDarwin
    @swiftTest
    def test(self):
        """
        Test flipping on/off implicit modules.
        """
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here",
                                          lldb.SBFileSpec('main.swift'))
        log = self.getBuildArtifact("types.log")
        self.expect('log enable lldb types -f "%s"' % log)
        self.expect("expression obj", DATA_TYPES_DISPLAYED_CORRECTLY,
                    substrs=["b ="])
        self.filecheck('platform shell cat "%s"' % log, __file__)
#       CHECK:  SwiftASTContextForExpressions(module: "a", cu: "main.swift")::LogConfiguration() --     -cc1



"""Test evaluating expressions repeatedly comparing lldb against gdb."""

import sys
import lldb
from lldbsuite.test.lldbbench import BenchBase
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import configuration
from lldbsuite.test import lldbutil


class RepeatedExprsCase(BenchBase):
    def setUp(self):
        BenchBase.setUp(self)
        self.source = "main.cpp"
        self.line_to_break = line_number(self.source, "// Set breakpoint here.")
        self.lldb_avg = None
        self.gdb_avg = None
        self.count = 100

    @benchmarks_test
    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="llvm.org/pr22274: need a pexpect replacement for windows",
    )
    def test_compare_lldb_to_gdb(self):
        """Test repeated expressions with lldb vs. gdb."""
        self.build()
        self.exe_name = "a.out"

        print()
        self.run_lldb_repeated_exprs(self.exe_name, self.count)
        print("lldb benchmark:", self.stopwatch)
        self.run_gdb_repeated_exprs(self.exe_name, self.count)
        print("gdb benchmark:", self.stopwatch)
        print("lldb_avg/gdb_avg: %f" % (self.lldb_avg / self.gdb_avg))

    def run_lldb_repeated_exprs(self, exe_name, count):
        import pexpect

        exe = self.getBuildArtifact(exe_name)

        # Set self.child_prompt, which is "(lldb) ".
        self.child_prompt = "(lldb) "
        prompt = self.child_prompt

        # So that the child gets torn down after the test.
        self.child = pexpect.spawn(
            "%s %s %s" % (lldbtest_config.lldbExec, self.lldbOption, exe)
        )
        child = self.child

        # Turn on logging for what the child sends back.
        if self.TraceOn():
            child.logfile_read = sys.stdout

        child.expect_exact(prompt)
        child.sendline("breakpoint set -f %s -l %d" % (self.source, self.line_to_break))
        child.expect_exact(prompt)
        child.sendline("run")
        child.expect_exact(prompt)
        expr_cmd1 = "expr ptr[j]->point.x"
        expr_cmd2 = "expr ptr[j]->point.y"

        # Reset the stopwatch now.
        self.stopwatch.reset()
        for i in range(count):
            with self.stopwatch:
                child.sendline(expr_cmd1)
                child.expect_exact(prompt)
                child.sendline(expr_cmd2)
                child.expect_exact(prompt)
            child.sendline("process continue")
            child.expect_exact(prompt)

        child.sendline("quit")
        try:
            self.child.expect(pexpect.EOF)
        except:
            pass

        self.lldb_avg = self.stopwatch.avg()
        if self.TraceOn():
            print("lldb expression benchmark:", str(self.stopwatch))
        self.child = None

    def run_gdb_repeated_exprs(self, exe_name, count):
        import pexpect

        exe = self.getBuildArtifact(exe_name)

        # Set self.child_prompt, which is "(gdb) ".
        self.child_prompt = "(gdb) "
        prompt = self.child_prompt

        # So that the child gets torn down after the test.
        self.child = pexpect.spawn("gdb --nx %s" % exe)
        child = self.child

        # Turn on logging for what the child sends back.
        if self.TraceOn():
            child.logfile_read = sys.stdout

        child.expect_exact(prompt)
        child.sendline("break %s:%d" % (self.source, self.line_to_break))
        child.expect_exact(prompt)
        child.sendline("run")
        child.expect_exact(prompt)
        expr_cmd1 = "print ptr[j]->point.x"
        expr_cmd2 = "print ptr[j]->point.y"

        # Reset the stopwatch now.
        self.stopwatch.reset()
        for i in range(count):
            with self.stopwatch:
                child.sendline(expr_cmd1)
                child.expect_exact(prompt)
                child.sendline(expr_cmd2)
                child.expect_exact(prompt)
            child.sendline("continue")
            child.expect_exact(prompt)

        child.sendline("quit")
        child.expect_exact("The program is running.  Exit anyway?")
        child.sendline("y")
        try:
            self.child.expect(pexpect.EOF)
        except:
            pass

        self.gdb_avg = self.stopwatch.avg()
        if self.TraceOn():
            print("gdb expression benchmark:", str(self.stopwatch))
        self.child = None



# TestSwiftStdlibDictionary.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Tests that we properly vend synthetic children for Swift.Dictionary
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftStdlibDictionary(TestBase):
    def get_variable(self, name):
        var = self.frame().FindVariable(
            name).GetDynamicValue(lldb.eDynamicCanRunTarget)
        var.SetPreferSyntheticValue(True)
        return var

    def find_dictionary_entry(
            self,
            vdict,
            key_summary=None,
            key_value=None,
            value_summary=None,
            value_value=None,
            fail_on_missing=True):
        self.assertTrue(vdict.IsValid(), "invalid Dictionary")
        count = vdict.GetNumChildren()
        found = False
        for i in range(0, count):
            child = vdict.GetChildAtIndex(i)
            if child.IsValid():
                key = child.GetChildMemberWithName("key")
                value = child.GetChildMemberWithName("value")

                key_match = False
                value_match = False

                if key_value:
                    if key.GetValue() == key_value:
                        key_match = True
                elif key_summary:
                    if key.GetSummary() == key_summary:
                        key_match = True

                if value_value:
                    if value.GetValue() == value_value:
                        value_match = True
                elif value_summary:
                    if value.GetSummary() == value_summary:
                        value_match = True

                if key_match and value_match:
                    found = True
                    break

        if key_value:
            key_str = str(key_value)
        else:
            key_str = str(key_summary)

        if value_value:
            value_str = str(value_value)
        else:
            value_str = str(value_summary)

        if fail_on_missing:
            self.assertTrue(
                found, ("could not find an expected child for '%s':'%s'" %
                        (key_str, value_str)))
        else:
            self.assertFalse(
                found, ("found a not expected child for '%s':'%s'" %
                        (key_str, value_str)))

    @swiftTest
    # @skipIfLinux  # bugs.swift.org/SR-844
    def test_swift_stdlib_dictionary(self):
        """Tests that we properly vend synthetic children for Swift.Dictionary"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type summary delete a.Wrapper", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.runCmd('type summary add a.Wrapper -s ${var.value%S}')

        for i in range(0, 100):
            self.find_dictionary_entry(
                self.get_variable("d"),
                key_value=str(i),
                value_summary='"%s"' % (i * 2 + 1))

        self.runCmd('expression d.removeValue(forKey: 34)')
        self.find_dictionary_entry(
            self.get_variable("d"),
            key_value=34,
            value_summary='"43"',
            fail_on_missing=False)




#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

from enum import Enum


class FailurePropagationMode(Enum):
    """Propagation mode for silenceable errors."""

    PROPAGATE = 1
    SUPPRESS = 2

    def _as_int(self):
        if self is FailurePropagationMode.PROPAGATE:
            return 1

        assert self is FailurePropagationMode.SUPPRESS
        return 2


from .._transform_ops_gen import *
from ..._mlir_libs._mlirDialectsTransform import *



"""Test that the Objective-C syntax for dictionary/array literals and indexing works"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

from ObjCNewSyntaxTest import ObjCNewSyntaxTest


class ObjCNewSyntaxTestCaseArray(ObjCNewSyntaxTest):
    @skipIf(macos_version=["<", "10.12"])
    @expectedFailureAll(archs=["i[3-6]86"])
    def test_read_array(self):
        self.runToBreakpoint()

        self.expect(
            "expr --object-description -- immutable_array[0]",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["foo"],
        )

        self.expect(
            "expr --object-description -- mutable_array[0]",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["foo"],
        )

    @skipIf(macos_version=["<", "10.12"])
    @expectedFailureAll(archs=["i[3-6]86"])
    def test_update_array(self):
        self.runToBreakpoint()

        self.expect(
            'expr --object-description -- mutable_array[0] = @"bar"',
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["bar"],
        )

        self.expect(
            "expr --object-description -- mutable_array[0]",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["bar"],
        )

    @skipIf(macos_version=["<", "10.12"])
    @expectedFailureAll(archs=["i[3-6]86"])
    def test_array_literal(self):
        self.runToBreakpoint()

        self.expect(
            'expr --object-description -- @[ @"foo", @"bar" ]',
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["NSArray", "foo", "bar"],
        )



"""
Test std::vector functionality when it's contents are vectors.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestVectorOfVectors(TestBase):
    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        if self.expectedCompiler(["clang"]) and self.expectedCompilerVersion(
            [">", "16.0"]
        ):
            vector_type = "std::vector<int>"
            vector_of_vector_type = "std::vector<std::vector<int> >"
        else:
            vector_type = "std::vector<int>"
            vector_of_vector_type = (
                "std::vector<std::vector<int>, std::allocator<std::vector<int> > >"
            )

        size_type = "size_type"
        value_type = "value_type"

        self.runCmd("settings set target.import-std-module true")

        self.expect_expr(
            "a",
            result_type=vector_of_vector_type,
            result_children=[
                ValueCheck(
                    type=vector_type,
                    children=[
                        ValueCheck(value="1"),
                        ValueCheck(value="2"),
                        ValueCheck(value="3"),
                    ],
                ),
                ValueCheck(
                    type=vector_type,
                    children=[
                        ValueCheck(value="3"),
                        ValueCheck(value="2"),
                        ValueCheck(value="1"),
                    ],
                ),
            ],
        )
        self.expect_expr("a.size()", result_type=size_type, result_value="2")
        front = self.expect_expr(
            "a.front().front()", result_type=value_type, result_value="1"
        )
        self.expect_expr("a[1][1]", result_type=value_type, result_value="2")
        self.expect_expr("a.back().at(0)", result_type=value_type, result_value="3")



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIfRemote
    def test_load_after_attach(self):
        self.build()

        sync_file_path = lldbutil.append_to_process_working_directory(
            self, "process_ready"
        )

        ctx = self.platformContext
        lib_name = ctx.shlib_prefix + "lib_b." + ctx.shlib_extension

        exe = self.getBuildArtifact("a.out")
        lib = self.getBuildArtifact(lib_name)

        target = self.dbg.CreateTarget(exe)
        environment = self.registerSharedLibrariesWithTarget(target, ["lib_b"])

        # Spawn a new process.
        # use realpath to workaround llvm.org/pr48376
        # Pass path to solib for dlopen to properly locate the library.
        popen = self.spawnSubprocess(
            os.path.realpath(exe), [sync_file_path], extra_env=environment
        )
        lldbutil.wait_for_file_on_target(self, sync_file_path)

        # Attach to the spawned process.
        error = lldb.SBError()
        process = target.AttachToProcessWithID(self.dbg.GetListener(), popen.pid, error)
        self.assertSuccess(error)

        # Continue until first breakpoint.
        breakpoint1 = self.target().BreakpointCreateBySourceRegex(
            "// break here", lldb.SBFileSpec("main.cpp")
        )
        self.assertEqual(breakpoint1.GetNumResolvedLocations(), 1)
        stopped_threads = lldbutil.continue_to_breakpoint(self.process(), breakpoint1)
        self.assertEqual(len(stopped_threads), 1)

        # Change a variable to escape the loop
        self.runCmd("expression main_thread_continue = 1")

        # Continue so that dlopen is called.
        breakpoint2 = self.target().BreakpointCreateBySourceRegex(
            "// break after dlopen", lldb.SBFileSpec("main.cpp")
        )
        self.assertEqual(breakpoint2.GetNumResolvedLocations(), 1)
        stopped_threads = lldbutil.continue_to_breakpoint(self.process(), breakpoint2)
        self.assertEqual(len(stopped_threads), 1)

        # Check that image list contains liblib_b after dlopen.
        self.match(
            "image list",
            patterns=[lib_name],
            matching=True,
            msg=lib_name + " missing in image list",
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import json


class TestSimulatorPlatformLaunching(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def check_load_commands(self, expected_load_command):
        """sanity check the built binary for the expected number of load commands"""
        load_cmds = subprocess.check_output(
            ["otool", "-l", self.getBuildArtifact()]
        ).decode("utf-8")
        found = 0
        for line in load_cmds.split("\n"):
            if expected_load_command in line:
                found += 1
        self.assertEqual(
            found,
            1,
            "wrong number of load commands for {}".format(expected_load_command),
        )

    def check_debugserver(self, log, expected_platform, expected_version):
        """scan the debugserver packet log"""
        process_info = lldbutil.packetlog_get_process_info(log)
        self.assertIn("ostype", process_info)
        self.assertEqual(process_info["ostype"], expected_platform)
        dylib_info = lldbutil.packetlog_get_dylib_info(log)
        self.assertTrue(dylib_info)
        aout_info = None
        for image in dylib_info["images"]:
            if image["pathname"].endswith("a.out"):
                aout_info = image
        self.assertTrue(aout_info)
        self.assertEqual(aout_info["min_version_os_name"], expected_platform)
        if expected_version:
            self.assertEqual(aout_info["min_version_os_sdk"], expected_version)

    @skipIf(bugnumber="rdar://76995109")
    def run_with(self, arch, os, vers, env, expected_load_command):
        env_list = [env] if env else []
        triple = "-".join([arch, "apple", os + vers] + env_list)
        sdk = lldbutil.get_xcode_sdk(os, env)

        version_min = ""
        if not vers:
            vers = lldbutil.get_xcode_sdk_version(sdk)
        if env == "simulator":
            version_min = "-m{}-simulator-version-min={}".format(os, vers)
        elif os == "macosx":
            version_min = "-m{}-version-min={}".format(os, vers)

        sdk_root = lldbutil.get_xcode_sdk_root(sdk)
        clang = lldbutil.get_xcode_clang(sdk)

        self.build(
            dictionary={
                "ARCH": arch,
                "CC": clang,
                "ARCH_CFLAGS": "-target {} {}".format(triple, version_min),
                "SDKROOT": sdk_root,
            }
        )

        self.check_load_commands(expected_load_command)
        log = self.getBuildArtifact("packets.log")
        self.expect("log enable gdb-remote packets -f " + log)
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("hello.c")
        )
        triple_re = "-".join([arch, "apple", os + vers + ".*"] + env_list)
        self.expect("image list -b -t", patterns=["a\.out " + triple_re])
        self.check_debugserver(log, os + env, vers)

    @skipIfAsan
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @apple_simulator_test("iphone")
    @skipIfOutOfTreeDebugserver
    def test_ios(self):
        """Test running an iOS simulator binary"""
        self.run_with(
            arch=self.getArchitecture(),
            os="ios",
            vers="",
            env="simulator",
            expected_load_command="LC_BUILD_VERSION",
        )

    @skipIfAsan
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @apple_simulator_test("appletv")
    @skipIfOutOfTreeDebugserver
    def test_tvos(self):
        """Test running an tvOS simulator binary"""
        self.run_with(
            arch=self.getArchitecture(),
            os="tvos",
            vers="",
            env="simulator",
            expected_load_command="LC_BUILD_VERSION",
        )

    @skipIfAsan
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @apple_simulator_test("watch")
    @skipIfDarwin  # rdar://problem/64552748
    @skipIf(archs=["arm64", "arm64e"])
    @skipIfOutOfTreeDebugserver
    def test_watchos_i386(self):
        """Test running a 32-bit watchOS simulator binary"""
        self.run_with(
            arch="i386",
            os="watchos",
            vers="",
            env="simulator",
            expected_load_command="LC_BUILD_VERSION",
        )

    @skipIfAsan
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @apple_simulator_test("watch")
    @skipIfDarwin  # rdar://problem/64552748
    @skipIf(archs=["i386", "x86_64"])
    @skipIfOutOfTreeDebugserver
    def test_watchos_armv7k(self):
        """Test running a 32-bit watchOS simulator binary"""
        self.run_with(
            arch="armv7k",
            os="watchos",
            vers="",
            env="simulator",
            expected_load_command="LC_BUILD_VERSION",
        )

    #
    # Back-deployment tests.
    #
    # Older Mach-O versions used less expressive load commands, such
    # as LC_VERSION_MIN_IPHONEOS that wouldn't distinguish between ios
    # and ios-simulator.  When targeting a simulator on Apple Silicon
    # macOS, however, these legacy load commands are never generated.
    #

    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @skipIfOutOfTreeDebugserver
    def test_lc_version_min_macosx(self):
        """Test running a back-deploying non-simulator MacOS X binary"""
        self.run_with(
            arch=self.getArchitecture(),
            os="macosx",
            vers="10.9",
            env="",
            expected_load_command="LC_VERSION_MIN_MACOSX",
        )

    @skipIfAsan
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @apple_simulator_test("iphone")
    @skipIf(archs=["arm64", "arm64e"])
    @skipIfOutOfTreeDebugserver
    def test_lc_version_min_iphoneos(self):
        """Test running a back-deploying iOS simulator binary
        with a legacy iOS load command"""
        self.run_with(
            arch=self.getArchitecture(),
            os="ios",
            vers="11.0",
            env="simulator",
            expected_load_command="LC_VERSION_MIN_IPHONEOS",
        )

    @skipIfAsan
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @apple_simulator_test("iphone")
    @skipIf(archs=["arm64", "arm64e"])
    @skipIfOutOfTreeDebugserver
    def test_ios_backdeploy_x86(self):
        """Test running a back-deploying iOS simulator binary
        with a legacy iOS load command"""
        self.run_with(
            arch=self.getArchitecture(),
            os="ios",
            vers="13.0",
            env="simulator",
            expected_load_command="LC_BUILD_VERSION",
        )

    @skipIfAsan
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @apple_simulator_test("iphone")
    @skipIf(archs=["i386", "x86_64"])
    @skipIfOutOfTreeDebugserver
    def test_ios_backdeploy_apple_silicon(self):
        """Test running a back-deploying iOS simulator binary"""
        self.run_with(
            arch=self.getArchitecture(),
            os="ios",
            vers="11.0",
            env="simulator",
            expected_load_command="LC_BUILD_VERSION",
        )

    @skipIfAsan
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @apple_simulator_test("appletv")
    @skipIf(archs=["arm64", "arm64e"])
    @skipIfOutOfTreeDebugserver
    def test_lc_version_min_tvos(self):
        """Test running a back-deploying tvOS simulator binary
        with a legacy tvOS load command"""
        self.run_with(
            arch=self.getArchitecture(),
            os="tvos",
            vers="11.0",
            env="simulator",
            expected_load_command="LC_VERSION_MIN_TVOS",
        )

    @skipIfAsan
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @apple_simulator_test("appletv")
    @skipIf(archs=["i386", "x86_64"])
    @skipIfOutOfTreeDebugserver
    def test_tvos_backdeploy_apple_silicon(self):
        """Test running a back-deploying tvOS simulator binary"""
        self.run_with(
            arch=self.getArchitecture(),
            os="tvos",
            vers="11.0",
            env="simulator",
            expected_load_command="LC_BUILD_VERSION",
        )

    @skipIfAsan
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @apple_simulator_test("watch")
    @skipIf(archs=["arm64", "arm64e"])
    @skipIfDarwin  # rdar://problem/64552748
    @skipIfOutOfTreeDebugserver
    def test_lc_version_min_watchos(self):
        """Test running a back-deploying watchOS simulator binary
        with a legacy watchOS load command"""
        self.run_with(
            arch="i386",
            os="watchos",
            vers="4.0",
            env="simulator",
            expected_load_command="LC_VERSION_MIN_WATCHOS",
        )

    @skipIfAsan
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @apple_simulator_test("watch")
    @skipIf(archs=["arm64", "arm64e"])
    @skipIfDarwin  # rdar://problem/64552748
    @skipIfOutOfTreeDebugserver
    def test_watchos_backdeploy_apple_silicon(self):
        """Test running a back-deploying watchOS simulator binary"""
        self.run_with(
            arch="armv7k",
            os="watchos",
            vers="4.0",
            env="simulator",
            expected_load_command="LC_BUILD_VERSION",
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftNestedGenericClass(TestBase):

    @swiftTest
    def test(self):
        """Tests that a generic class type nested inside another generic class can be resolved correctly from the instance metadata"""
        self.build()
        _, process, _, breakpoint = lldbutil.run_to_source_breakpoint(self, 
                                "break here", lldb.SBFileSpec("main.swift"))
        self.expect("v self", substrs=["A<Int>.B<String>"])

        lldbutil.continue_to_breakpoint(process, breakpoint)
        self.expect("v self", substrs=["C.D<Double>"])

        lldbutil.continue_to_breakpoint(process, breakpoint)
        self.expect("v self", substrs=["F.G<Bool>.H"])

        lldbutil.continue_to_breakpoint(process, breakpoint)
        self.expect("v self", substrs=["I.J<String, Int>"])

        
        lldbutil.continue_to_breakpoint(process, breakpoint)
        self.expect("v self", substrs=["K.L.M<Double, Bool>"])




"""
Test the 'memory write' command.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class MemoryWriteTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.line = line_number("main.c", "// Set break point at this line.")

    def build_run_stop(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break in main() after the variables are assigned values.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

    @no_debug_info_test
    def test_memory_write(self):
        """Test the 'memory write' command for writing values and file contents."""
        self.build_run_stop()

        self.expect(
            "memory read --format c --size 7 --count 1 `&my_string`",
            substrs=["abcdefg"],
        )

        self.expect("memory write --format c --size 7 `&my_string` ABCDEFG")

        self.expect(
            "memory read --format c --size 7 --count 1 `&my_string`",
            substrs=["ABCDEFG"],
        )

        self.expect(
            "memory write --infile file.txt --size 7 `&my_string`",
            substrs=["7 bytes were written"],
        )

        self.expect(
            "memory read --format c --size 7 --count 1 `&my_string`",
            substrs=["abcdefg"],
        )

        self.expect(
            "memory write --infile file.txt --size 7 `&my_string` ABCDEFG",
            error=True,
            substrs=[
                "error: memory write takes only a destination address when writing file contents"
            ],
        )

        self.expect(
            "memory write --infile file.txt --size 7",
            error=True,
            substrs=[
                "error: memory write takes a destination address when writing file contents"
            ],
        )

    @no_debug_info_test
    def test_memory_write_command_usage_syntax(self):
        """Test that 'memory write' command usage syntax shows it does not take values when writing file contents."""
        self.expect(
            "help memory write",
            substrs=[
                "memory write [-f <format>] [-s <byte-size>] <address> <value> [<value> [...]]",
                "memory write -i <filename> [-s <byte-size>] [-o <offset>] <address>",
            ],
        )



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import re

class TestCase(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    @swiftTest
    @skipIf(oslist=['windows', 'linux'])
    @skipIf(bugnumber="rdar://116529018")
    def test(self):
        """Test step-in to async functions"""
        self.build()
        src = lldb.SBFileSpec('main.swift')
        _, process, _, _ = lldbutil.run_to_source_breakpoint(self, 'await', src)

        # When run with debug info enabled builds, this prevents stepping from
        # stopping in Swift Concurrency runtime functions.
        self.runCmd("settings set target.process.thread.step-avoid-libraries libswift_Concurrency.dylib")

        # All thread actions are done on the currently selected thread.
        thread = process.GetSelectedThread

        num_async_steps = 0
        while True:
            stop_reason = thread().stop_reason
            if stop_reason == lldb.eStopReasonNone:
                break
            elif stop_reason == lldb.eStopReasonPlanComplete:
                # Run until the next `await` breakpoint.
                process.Continue()
            elif stop_reason == lldb.eStopReasonBreakpoint:
                caller_before = thread().frames[0].function.GetDisplayName()
                line_before = thread().frames[0].line_entry.line
                thread().StepInto()
                caller_after = thread().frames[1].function.GetDisplayName()
                line_after = thread().frames[0].line_entry.line

                # Breakpoints on lines with an `await` may result in more than
                # one breakpoint location. Specifically a location before an
                # async function is called, and then a location on the resume
                # function. In this case, running `step` on these lines will
                # move execution forward within the same function, _not_ step
                # into a new function.
                #
                # As this test is for stepping into async functions, when the
                # step-in keeps execution on the same or next line -- not a
                # different function, then it can be ignored. rdar://76116620
                if line_after in (line_before, line_before + 1):
                    # When stepping stops at breakpoint, don't continue.
                    if thread().stop_reason != lldb.eStopReasonBreakpoint:
                        process.Continue()
                    continue

                # The entry function is missing this prefix dedicating resume functions.
                prefix = re.compile(r'^\([0-9]+\) await resume partial function for ')
                self.assertEqual(prefix.sub('', caller_after),
                                 prefix.sub('', caller_before))
                num_async_steps += 1

        self.assertGreater(num_async_steps, 0)



# TestSwiftCrossModuleExtension.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test that we correctly find private extension decls across modules
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import os.path
import unittest2

class TestSwiftCrossModuleExtension(TestBase):
    @skipUnlessDarwin
    @swiftTest
    def test_cross_module_extension(self):
        """Test that we correctly find private extension decls across modules"""
        self.build()
        target, process, thread, a_breakpoint = \
            lldbutil.run_to_source_breakpoint(
                self, 'break here', lldb.SBFileSpec('moda.swift'),
                exe_name = self.getBuildArtifact("main"))
        b_breakpoint = target.BreakpointCreateBySourceRegex(
            'break here', lldb.SBFileSpec('modb.swift'))
        self.assertTrue(b_breakpoint.GetNumLocations() > 0, VALID_BREAKPOINT)
        frame = thread.frames[0]
        self.assertTrue(frame, "Frame 0 is valid.")

        var = frame.FindVariable("a")
        child_v = var.GetChildMemberWithName("v")
        lldbutil.check_variable(self, var, False, typename="moda.S.A")
        lldbutil.check_variable(self, child_v, False, value="1")

        process.Continue()
        threads = lldbutil.get_threads_stopped_at_breakpoint(
            process, b_breakpoint)

        self.assertTrue(len(threads) == 1)
        frame = threads[0].frames[0]
        self.assertTrue(frame, "Frame 0 is valid.")

        var = frame.FindVariable("a")
        child_v = var.GetChildMemberWithName("v")
        lldbutil.check_variable(self, var, False, typename="moda.S.A")
        lldbutil.check_variable(self, child_v, False, value="3")





import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest

class SwiftCompletionTest(PExpectTest):
    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipUnlessDarwin
    @swiftTest
    def test_basic_completion(self):

        self.launch(extra_args=["--repl"], executable=None, dimensions=(100,500))

        # Wait on the first prompt
        self.child.expect_exact("1>")
        # Press tab a few times which should do nothing.
        # Note that we don't get any indentation whitespace as
        # pexpect is not recognized as a interactive terminal by pexpect it seems.
        self.child.send("\t\t\t")

        # Try completing something that only has one result "fun" -> "func".
        self.child.send("fun\t")
        self.child.expect_exact("func")
        self.child.sendline("")

        # Try completing something that has multiple completions.
        self.child.send("Hash\t")
        self.child.expect_exact("Available completions:")
        self.child.expect_exact("Hashable")
        self.child.expect_exact("Hasher")
        self.child.sendline("")

        self.quit()

    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIf(oslist=['windows'])
    @swiftTest
    def test_lldb_command_completion(self):

        self.launch(extra_args=["--repl"], executable=None, dimensions=(100,500))

        # Wait on the first prompt
        self.child.expect_exact("1>")

        # Try completing something already complete.
        self.child.send(":b\t")
        # Use list of strings to work when there are embedded ansi sequences.
        self.child.expect_exact([":", "b "])
        self.child.sendline("")

        # Try completing something that only has one result "vers" -> "version".
        self.child.send(":vers\t")
        self.child.expect_exact([":", "version"])
        self.child.sendline("")

        # Try completing something that has multiple completions.
        self.child.send(":\t")
        self.child.expect_exact("Available completions:")
        self.child.expect_exact([":", "help"])
        self.child.expect_exact("More (Y/n/a)")
        self.child.send("n")
        self.child.sendline("help")

        # Try completing something with subcommands.
        self.child.send(":breakpoi\t")
        # Use list of strings to work when there are embedded ansi sequences.
        self.child.expect_exact([":", "breakpoint "])
        self.child.send("\t")
        self.child.expect_exact("Available completions:")
        self.child.expect_exact("command")
        self.child.send("comm\t")
        # Use list of strings to work when there are embedded ansi sequences.
        self.child.expect_exact([":", "breakpoint ", "command "])
        self.child.send("li\t")
        # Use list of strings to work when there are embedded ansi sequences.
        self.child.expect_exact([":", "breakpoint ", "command ", "list"])
        self.child.sendline("")

        self.quit()

    def setUpCommands(self):
        return [] # REPL doesn't take any setup commands.

    def expect_prompt(self):
        pass # No constant prompt on the REPL.



import os
import re
import operator

import lit.Test
import lit.TestRunner
import lit.util
from lit.formats.base import TestFormat


class LLDBTest(TestFormat):
    def __init__(self, dotest_cmd):
        self.dotest_cmd = dotest_cmd

    def getTestsInDirectory(self, testSuite, path_in_suite, litConfig, localConfig):
        source_path = testSuite.getSourcePath(path_in_suite)
        for filename in os.listdir(source_path):
            # Ignore dot files and excluded tests.
            if filename.startswith(".") or filename in localConfig.excludes:
                continue

            # Ignore files that don't start with 'Test'.
            if not filename.startswith("Test"):
                continue

            filepath = os.path.join(source_path, filename)
            if not os.path.isdir(filepath):
                base, ext = os.path.splitext(filename)
                if ext in localConfig.suffixes:
                    yield lit.Test.Test(
                        testSuite, path_in_suite + (filename,), localConfig
                    )

    def execute(self, test, litConfig):
        if litConfig.noExecute:
            return lit.Test.PASS, ""

        if not getattr(test.config, "lldb_enable_python", False):
            return (lit.Test.UNSUPPORTED, "Python module disabled")

        if test.config.unsupported:
            return (lit.Test.UNSUPPORTED, "Test is unsupported")

        testPath, testFile = os.path.split(test.getSourcePath())

        # The Python used to run lit can be different from the Python LLDB was
        # build with.
        executable = test.config.python_executable

        isLuaTest = testFile == test.config.lua_test_entry

        # On Windows, the system does not always correctly interpret
        # shebang lines.  To make sure we can execute the tests, add
        # python exe as the first parameter of the command.
        cmd = [executable] + self.dotest_cmd + [testPath, "-p", testFile]

        if isLuaTest:
            luaExecutable = test.config.lua_executable
            cmd.extend(["--env", "LUA_EXECUTABLE=%s" % luaExecutable])

        timeoutInfo = None
        try:
            out, err, exitCode = lit.util.executeCommand(
                cmd,
                env=test.config.environment,
                timeout=litConfig.maxIndividualTestTime,
            )
        except lit.util.ExecuteCommandTimeoutException as e:
            out = e.out
            err = e.err
            exitCode = e.exitCode
            timeoutInfo = "Reached timeout of {} seconds".format(
                litConfig.maxIndividualTestTime
            )

        output = """Script:\n--\n%s\n--\nExit Code: %d\n""" % (" ".join(cmd), exitCode)
        if timeoutInfo is not None:
            output += """Timeout: %s\n""" % (timeoutInfo,)
        output += "\n"

        if out:
            output += """Command Output (stdout):\n--\n%s\n--\n""" % (out,)
        if err:
            output += """Command Output (stderr):\n--\n%s\n--\n""" % (err,)

        if timeoutInfo:
            return lit.Test.TIMEOUT, output

        # Temporary fix to a flaky CI error. See rdar://52221547
        # After unit tests have finished, a race during lldb shutdown can
        # result in this innocuous and trouble causing failure:
        #
        # libc++abi.dylib: terminating with uncaught exception of type \
        # std::__1::system_error: recursive_mutex lock failed: Invalid argument
        if (
            'RESULT: PASSED' in err and
            'recursive_mutex lock failed' in err and
            exitCode != 0
        ):
            return lit.Test.FLAKYPASS, output

        # Parse the dotest output from stderr.
        result_regex = r"\((\d+) passes, (\d+) failures, (\d+) errors, (\d+) skipped, (\d+) expected failures, (\d+) unexpected successes\)"
        results = re.search(result_regex, err)

        # If parsing fails mark this test as unresolved.
        if not results:
            return lit.Test.UNRESOLVED, output

        passes = int(results.group(1))
        failures = int(results.group(2))
        errors = int(results.group(3))
        skipped = int(results.group(4))
        expected_failures = int(results.group(5))
        unexpected_successes = int(results.group(6))

        if exitCode:
            # Mark this test as FAIL if at least one test failed.
            if failures > 0:
                return lit.Test.FAIL, output
            lit_results = [
                (failures, lit.Test.FAIL),
                (errors, lit.Test.UNRESOLVED),
                (unexpected_successes, lit.Test.XPASS),
            ]
        else:
            # Mark this test as PASS if at least one test passed.
            if passes > 0:
                return lit.Test.PASS, output
            lit_results = [
                (passes, lit.Test.PASS),
                (skipped, lit.Test.UNSUPPORTED),
                (expected_failures, lit.Test.XFAIL),
            ]

        # Return the lit result code with the maximum occurrence. Only look at
        # the first element and rely on the original order to break ties.
        return max(lit_results, key=operator.itemgetter(0))[1], output



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftArtificialSubclass(TestBase):
    @skipUnlessObjCInterop
    @swiftTest
    def test(self):
        """ Test that displaying an artificial type works correctly"""
        self.build()
        _, _, _, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift")
        )

        
        self.expect(
            "frame variable m",
            substrs=["Subclass)", "Superclass", "a = 42", "b = 97"]
        )



"""
Tests that ObjC member variables are available where they should be.
"""
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ObjCSelfTestCase(TestBase):
    def test_with_run_command(self):
        """Test that the appropriate member variables are available when stopped in Objective-C class and instance methods"""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        self.set_breakpoint(line_number("main.m", "// breakpoint 1"))
        self.set_breakpoint(line_number("main.m", "// breakpoint 2"))

        self.runCmd("process launch", RUN_SUCCEEDED)

        self.expect("expression -- m_a = 2", startstr="(int) $0 = 2")

        self.runCmd("process continue")

        # This would be disallowed if we enforced const.  But we don't.
        self.expect("expression -- m_a = 2", error=True)

        self.expect("expression -- s_a", startstr="(int) $1 = 5")

    def set_breakpoint(self, line):
        lldbutil.run_break_set_by_file_and_line(
            self, "main.m", line, num_expected_locations=1, loc_exact=True
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestFunctionNameWithoutArgs(TestBase):
    @skipUnlessDarwin
    @no_debug_info_test
    def test_function_name_without_args(self):
        self.build()
        target = self.createTestTarget()
        target.LaunchSimple(None, None, self.get_process_working_directory())

        self.runCmd("run", RUN_SUCCEEDED)
        self.expect(
            "bt",
            substrs=[
                "stop reason = hit program assert",
                "libsystem_kernel.dylib`__pthread_kill",
            ],
        )
        self.runCmd(
            'settings set frame-format "frame #${frame.index}: ${function.name-without-args}\n"'
        )
        self.expect(
            "bt",
            substrs=["stop reason = hit program assert", "frame #0: __pthread_kill"],
        )



# TestSwiftModuleSearchPaths.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Tests that we can import modules located using target.swift-module-search-paths
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2

class TestSwiftModuleSearchPaths(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.main_source = "main.swift"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)


    @swiftTest
    def test_swift_module_search_paths(self):
        """
        Tests that we can import modules located using
        target.swift-module-search-paths
        """
        
        # Build and run the dummy target
        self.build()
        
        exe_name = "a.out"
        exe = self.getBuildArtifact(exe_name)

        lldbutil.run_to_source_breakpoint(self, "break here",
                                          lldb.SBFileSpec('main.swift'))

        # Add the current working dir to the swift-module-search-paths
        self.runCmd("settings append target.swift-module-search-paths " +
                    self.getBuildDir())
        
        # import the module
        self.runCmd("e import Module")
        
        # Check that we know about the function declared in the module
        self.match(
            "e plusTen(10)", "error: Couldn't lookup symbols:", error=True)




# TestSwiftDeploymentTarget.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2018 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------

import lldbsuite.test.lldbinline as lldbinline

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2
import os


class TestSwiftDeploymentTarget(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessDarwin
    @skipIfDarwinEmbedded # This test uses macOS triples explicitly.
    @skipIf(macos_version=["<", "11.1"])
    @swiftTest
    def test_swift_deployment_target(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self,
                                          "break here",
                                          lldb.SBFileSpec('main.swift'))
        self.expect("expression f", substrs=['i = 23'])

    @skipUnlessDarwin
    @skipIfDarwinEmbedded # This test uses macOS triples explicitly.
    @skipIf(macos_version=["<", "11.1"])
    @swiftTest
    def test_swift_deployment_target_dlopen(self):
        self.build()
        target, process, _, _, = lldbutil.run_to_name_breakpoint(
            self, 'main', exe_name="dlopen_module")
        bkpt = target.BreakpointCreateBySourceRegex(
            'break here', lldb.SBFileSpec('NewerTarget.swift'))
        lldbutil.continue_to_breakpoint(process, bkpt)
        self.expect("expression self", substrs=['i = 23'])

    @skipUnlessDarwin
    @skipIfDarwinEmbedded # This test uses macOS triples explicitly.
    @skipIf(macos_version=["<", "11.1"])
    # FIXME: This config started failing in CI only after switching to
    # the query-based FindTypes API.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @swiftTest
    def test_swift_deployment_target_from_macho(self):
        self.build(dictionary={"MAKE_DSYM": "NO"})
        os.unlink(self.getBuildArtifact("a.swiftmodule"))
        log = self.getBuildArtifact("types.log")
        self.runCmd('log enable lldb types -f "%s"' % log)
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift")
        )
        self.expect("expression f", substrs=["i = 23"])
        self.filecheck('platform shell cat ""%s"' % log, __file__)
#       CHECK: SwiftASTContextForExpressions::SetTriple({{.*}}apple-macosx11.0.0
#       CHECK-NOT: SwiftASTContextForExpressions::RegisterSectionModules("a.out"){{.*}} AST Data blobs

    @skipUnlessDarwin  # This test uses macOS triples explicitly.
    @skipIfDarwinEmbedded
    @skipIf(macos_version=["<", "11.1"])
    @skipIf(setting=("symbols.swift-precise-compiler-invocation", "false"))
    @swiftTest
    def test_swift_precise_compiler_invocation_triple(self):
        """
        Ensure expressions prefer the target triple of their module, as it may
        differ from the target triple of the target. This is necessary for
        explicitly built modules.
        """
        self.build()
        log = self.getBuildArtifact("types.log")
        self.runCmd(f'log enable lldb types -f "{log}"')
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("NewerTarget.swift")
        )
        self.expect(
            "image list -t libNewerTarget.dylib",
            substrs=["-apple-macosx11.1.0"],
        )
        self.expect("expression self", substrs=["i = 23"])
        self.filecheck(
            f'platform shell cat "{log}"', __file__, "-check-prefix=CHECK-PRECISE"
        )
#       CHECK-PRECISE: SwiftASTContextForExpressions(module: "NewerTarget", cu: "NewerTarget.swift")::CreateInstance() -- Fully specified target triple {{.*}}-apple-macosx11.1.0
#       CHECK-PRECISE: SwiftASTContextForExpressions(module: "NewerTarget", cu: "NewerTarget.swift")::SetTriple("{{.*}}-apple-macosx11.1.0")



"""
Test some lldb platform commands.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class PlatformCommandTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @no_debug_info_test
    def test_help_platform(self):
        self.runCmd("help platform")

    @no_debug_info_test
    def test_help_shell_alias(self):
        self.expect(
            "help shell",
            substrs=[
                "Run a shell command on the host.",
                "shell <shell-command>",
                "'shell' is an abbreviation",
            ],
        )
        # "platform shell" has options. The "shell" alias for it does not.
        self.expect("help shell", substrs=["Command Options:"], matching=False)

    @no_debug_info_test
    def test_list(self):
        self.expect("platform list", patterns=["^Available platforms:"])

    @no_debug_info_test
    def test_process_list(self):
        self.expect("platform process list", substrs=["PID", "TRIPLE", "NAME"])

    @no_debug_info_test
    def test_process_info_with_no_arg(self):
        """This is expected to fail and to return a proper error message."""
        self.expect(
            "platform process info",
            error=True,
            substrs=["one or more process id(s) must be specified"],
        )

    @no_debug_info_test
    def test_status(self):
        self.expect(
            "platform status",
            substrs=[
                "Platform",
                "Triple",
                "OS Version",
                "Hostname",
                "Kernel",
            ],
        )

    @expectedFailureAll(oslist=["windows"])
    @no_debug_info_test
    def test_shell(self):
        """Test that the platform shell command can invoke ls."""
        triple = self.dbg.GetSelectedPlatform().GetTriple()
        if re.match(".*-.*-windows", triple):
            self.expect("platform shell dir c:\\", substrs=["Windows", "Program Files"])
            self.expect("shell dir c:\\", substrs=["Windows", "Program Files"])
        elif re.match(".*-.*-.*-android", triple):
            self.expect("platform shell ls /", substrs=["cache", "dev", "system"])
            self.expect("shell ls /", substrs=["cache", "dev", "system"])
        else:
            self.expect("platform shell ls /", substrs=["dev", "tmp", "usr"])
            self.expect("shell ls /", substrs=["dev", "tmp", "usr"])

    @no_debug_info_test
    def test_shell_builtin(self):
        """Test a shell built-in command (echo)"""
        self.expect("platform shell echo hello lldb", substrs=["hello lldb"])
        self.expect("shell echo hello lldb", substrs=["hello lldb"])

    @no_debug_info_test
    def test_shell_timeout(self):
        """Test a shell built-in command (sleep) that times out"""
        self.skipTest("Alias with option not supported by the command interpreter.")
        self.expect(
            "platform shell -t 1 -- sleep 15",
            error=True,
            substrs=["error: timed out waiting for shell command to complete"],
        )
        self.expect(
            "shell -t 1 --  sleep 3",
            error=True,
            substrs=["error: timed out waiting for shell command to complete"],
        )

    @no_debug_info_test
    @skipIfRemote
    def test_host_shell_interpreter(self):
        """Test the host platform shell with a different interpreter"""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.expect(
            "platform shell -h -s " + exe + " -- 'echo $0'",
            substrs=["SUCCESS", "a.out"],
        )



# TestSwiftObjCMainConflictingDylibs.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2018 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2
import shutil

class TestSwiftObjCMainConflictingDylibs(TestBase):
    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipUnlessDarwin
    @swiftTest
    def test(self):
        # To ensure we hit the rebuild problem remove the cache to avoid caching.
        mod_cache = self.getBuildArtifact("my-clang-modules-cache")
        if os.path.isdir(mod_cache):
          shutil.rmtree(mod_cache)

        self.runCmd('settings set symbols.clang-modules-cache-path "%s"'
                    % mod_cache)
        self.build()
        target, process, _, foo_breakpoint = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('Foo.swift'),
            extra_images=['Foo', 'Bar'])

        # Prime the module cache with the Foo variant.
        self.expect("fr var baz", "correct baz", substrs=["i_am_from_Foo"])
        self.assertTrue(os.path.isdir(mod_cache), "module cache exists")

        bar_breakpoint = target.BreakpointCreateBySourceRegex(
            'break here', lldb.SBFileSpec('Bar.swift'))
        
        # Restart.
        process = target.LaunchSimple(None, None, os.getcwd())
        # This is failing because the Target-SwiftASTContext uses the
        # amalgamated target header search options from all dylibs.
        self.expect("expression baz", "wrong baz", substrs=["i_am_from_Foo"])
        self.expect("fr var baz", "wrong baz", substrs=["i_am_from_Foo"])


        process.Continue()
        self.expect("expression baz", "correct baz", substrs=["i_am_from_Foo"])
        self.expect("fr var baz", "correct baz", substrs=["i_am_from_Foo"])
        



"""
Use lldb Python SBTarget API to iterate on the watchpoint(s) for the target.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class WatchpointIteratorTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    # hardware watchpoints are not reported with a hardware index # on armv7 on ios devices
    def affected_by_radar_34564183(self):
        return (
            self.getArchitecture() in ["armv7", "armv7k", "arm64_32"]
        ) and self.platformIsDarwin()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Our simple source filename.
        self.source = "main.c"
        # Find the line number to break inside main().
        self.line = line_number(self.source, "// Set break point at this line.")

    def test_watch_iter(self):
        """Exercise SBTarget.watchpoint_iter() API to iterate on the available watchpoints."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Create a breakpoint on main.c in order to set our watchpoint later.
        breakpoint = target.BreakpointCreateByLocation(self.source, self.line)
        self.assertTrue(
            breakpoint and breakpoint.GetNumLocations() == 1, VALID_BREAKPOINT
        )

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        # We should be stopped due to the breakpoint.  Get frame #0.
        process = target.GetProcess()
        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        frame0 = thread.GetFrameAtIndex(0)

        # Watch 'global' for read and write.
        value = frame0.FindValue("global", lldb.eValueTypeVariableGlobal)
        error = lldb.SBError()
        watchpoint = value.Watch(True, False, True, error)
        self.assertTrue(
            value and watchpoint, "Successfully found the variable and set a watchpoint"
        )
        self.DebugSBValue(value)

        # Hide stdout if not running with '-t' option.
        if not self.TraceOn():
            self.HideStdout()

        # There should be only 1 watchpoint location under the target.
        self.assertEqual(target.GetNumWatchpoints(), 1)
        self.assertTrue(watchpoint.IsEnabled())
        watch_id = watchpoint.GetID()
        self.assertTrue(watch_id != 0)

        # Continue.  Expect the program to stop due to the variable being
        # written to.
        process.Continue()

        # Hide stdout if not running with '-t' option.
        if not self.TraceOn():
            self.HideStdout()

        # Print the stack traces.
        lldbutil.print_stacktraces(process)

        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonWatchpoint)
        self.assertTrue(thread, "The thread stopped due to watchpoint")
        self.DebugSBValue(value)

        # We currently only support hardware watchpoint.  Verify that we have a
        # meaningful hardware index at this point.  Exercise the printed repr of
        # SBWatchpointLocation.
        print(watchpoint)
        if not self.affected_by_radar_34564183():
            self.assertTrue(watchpoint.GetHardwareIndex() != -1)

        # SBWatchpoint.GetDescription() takes a description level arg.
        print(lldbutil.get_description(watchpoint, lldb.eDescriptionLevelFull))

        # Now disable the 'rw' watchpoint.  The program won't stop when it reads
        # 'global' next.
        watchpoint.SetEnabled(False)
        self.assertEqual(watchpoint.GetHardwareIndex(), -1)
        self.assertFalse(watchpoint.IsEnabled())

        # Continue.  The program does not stop again when the variable is being
        # read from because the watchpoint location has been disabled.
        process.Continue()

        # At this point, the inferior process should have exited.
        self.assertEqual(process.GetState(), lldb.eStateExited, PROCESS_EXITED)

        # Verify some vital statistics and exercise the iterator API.
        for watchpoint in target.watchpoint_iter():
            self.assertTrue(watchpoint)
            self.assertEqual(watchpoint.GetWatchSize(), 4)
            self.assertEqual(watchpoint.GetHitCount(), 1)
            print(watchpoint)



# TestEnumVariables.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Tests that Enum variables display correctly
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestEnumVariables(TestBase):
    @swiftTest
    def test_enum_variables(self):
        """Tests that Enum variables display correctly"""
        self.build()
        self.do_test()

    def setUp(self):
        TestBase.setUp(self)
        self.main_source = "main.swift"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)

    def get_variable(self, name):
        return self.frame().FindVariable(
            name).GetDynamicValue(lldb.eDynamicCanRunTarget)

    def check_enum(
            self,
            var_name,
            value,
            child_summary=None,
            child_value=None):
        var = self.frame().FindVariable(var_name)
        self.assertTrue(var.IsValid(), "invalid variable")
        self.assertTrue(var.GetValue() == value, "invalid value")
        if child_summary:
            child_var = var.GetChildMemberWithName(value)
            self.assertTrue(child_var.IsValid(), "invalid child")
            if child_summary:
                self.assertTrue(
                    child_var.GetSummary() == child_summary,
                    "invalid child summary")
            if child_value:
                self.assertTrue(
                    child_var.GetValue() == child_value,
                    "invalid child value")

    def do_test(self):
        """Tests that Enum variables display correctly"""
        lldbutil.run_to_source_breakpoint(self, 'Set breakpoint here',
                                          lldb.SBFileSpec('main.swift'))

        self.check_enum("ona", "A")
        self.check_enum("onb", "B")
        self.check_enum("onc", "C")
        self.check_enum("ond", "D")

        self.check_enum("twa", "A")
        self.check_enum("twb", "B", '"hello world"')
        self.check_enum("twc", "C", child_value='12')
        self.check_enum("twd", "D")

        self.check_enum("tha", "A", '"hello world"')
        self.check_enum("thb", "B", child_value='24')
        self.check_enum("thc", "C", '"this is me"')
        self.check_enum("thd", "D", "true")

        self.check_enum("foa", "A", '"hello world"')
        self.check_enum("fob", "B", '"this is me"')
        self.check_enum("foc", "C", '"life should be"')
        self.check_enum("fod", "D", '"fun for everyone"')

        self.expect('frame variable ContainerOfEnums_Some',
                    substrs=['Some', 'one1 = A', 'one2 = A'])
        self.expect('frame variable ContainerOfEnums_Nil', substrs=['nil'])




"""
Test that the type of arguments to C++ template classes that have variadic
parameters can be enumerated.
"""
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TemplatePackArgsTestCase(TestBase):
    def test_template_argument_pack(self):
        self.build()
        (_, _, thread, _) = lldbutil.run_to_source_breakpoint(
            self, "breakpoint here", lldb.SBFileSpec("main.cpp"), exe_name="a.out"
        )
        frame = thread.GetSelectedFrame()

        empty_pack = frame.FindVariable("emptyPack")
        self.assertTrue(
            empty_pack.IsValid(), "make sure we find the emptyPack variable"
        )

        only_pack = frame.FindVariable("onlyPack")
        self.assertTrue(only_pack.IsValid(), "make sure we find the onlyPack variable")
        self.assertEqual(only_pack.GetType().GetNumberOfTemplateArguments(), 4)
        self.assertEqual(
            only_pack.GetType().GetTemplateArgumentType(0).GetName(), "int"
        )
        self.assertEqual(
            only_pack.GetType().GetTemplateArgumentType(1).GetName(), "char"
        )
        self.assertEqual(
            only_pack.GetType().GetTemplateArgumentType(2).GetName(), "double"
        )
        # Access the C<double, 42> template parameter.
        nested_template = only_pack.GetType().GetTemplateArgumentType(3)
        self.assertEqual(nested_template.GetName(), "D<int, int, bool>")
        self.assertEqual(nested_template.GetNumberOfTemplateArguments(), 3)
        self.assertEqual(nested_template.GetTemplateArgumentType(0).GetName(), "int")
        self.assertEqual(nested_template.GetTemplateArgumentType(1).GetName(), "int")
        self.assertEqual(nested_template.GetTemplateArgumentType(2).GetName(), "bool")




"""
Test that a C++ class is visible in Swift.
"""
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestSwiftForwardInteropCxxLangOpt(TestBase):

    @swiftTest
    @expectedFailureAll(setting=('symbols.swift-precise-compiler-invocation', 'false'))
    def test_class(self):
        """
        Test that C++ interoperability is enabled on a per-CU basis.
        """
        self.build()
        log = self.getBuildArtifact("types.log")
        self.runCmd('log enable lldb types -f "%s"' % log)
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'),
            extra_images=['Dylib'])
        dylib_bkpt = target.BreakpointCreateBySourceRegex(
            'break here', lldb.SBFileSpec('Dylib.swift'))
        self.assertGreater(dylib_bkpt.GetNumLocations(), 0, VALID_BREAKPOINT)
        self.expect('expr 0')
        lldbutil.continue_to_breakpoint(process, dylib_bkpt)
        self.expect('expr 1')
        self.filecheck('platform shell cat ""%s"' % log, __file__)
#       CHECK: SwiftASTContextForExpressions(module: "a", cu: "main.swift")::LogConfiguration(){{.*}}Swift/C++ interop : on
#       CHECK:  SwiftASTContextForExpressions(module: "Dylib", cu: "Dylib.swift")::LogConfiguration(){{.*}}Swift/C++ interop : off




"""
Test that we can have persistent pointer variables
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class PersistentPtrUpdateTestCase(TestBase):
    def test(self):
        """Test that we can have persistent pointer variables"""
        self.build()

        def cleanup():
            pass

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        self.runCmd("break set -p here")

        self.runCmd("run", RUN_SUCCEEDED)

        self.runCmd("expr void* $foo = 0")

        self.runCmd("continue")

        self.expect("expr $foo", substrs=["$foo", "0x0"])



"""Test that lldb recognizes enum structs emitted by Rust compiler """
import logging

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from RustEnumValue import RustEnumValue


class TestRustEnumStructs(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        src_dir = self.getSourceDir()
        yaml_path = os.path.join(src_dir, "main.yaml")
        obj_path = self.getBuildArtifact("main.o")
        self.yaml2obj(yaml_path, obj_path)
        self.dbg.CreateTarget(obj_path)

    def getFromGlobal(self, name):
        values = self.target().FindGlobalVariables(name, 1)
        self.assertEqual(values.GetSize(), 1)
        return RustEnumValue(values[0])

    def test_clike_enums_are_represented_correctly(self):
        # these type of enums are not using DW_TAG_variant_part.
        all_values = [
            self.target().FindFirstGlobalVariable("CLIKE_DEFAULT_A").GetValue(),
            self.target().FindFirstGlobalVariable("CLIKE_DEFAULT_B").GetValue(),
            self.target().FindFirstGlobalVariable("CLIKE_U8_A").GetValue(),
            self.target().FindFirstGlobalVariable("CLIKE_U8_C").GetValue(),
            self.target().FindFirstGlobalVariable("CLIKE_U32_A").GetValue(),
            self.target().FindFirstGlobalVariable("CLIKE_U32_B").GetValue(),
        ]
        self.assertEqual(all_values, ['A', 'B', 'VariantA', 'VariantC', 'VariantA', 'VariantB'])

    def test_enum_with_tuples_has_all_variants(self):
        self.assertEqual(self.getFromGlobal("ENUM_WITH_TUPLES_A").getAllVariantTypes(),
                         ['main::EnumWithTuples::A:8',
                          'main::EnumWithTuples::B:8',
                          'main::EnumWithTuples::C:8',
                          'main::EnumWithTuples::D:8',
                          'main::EnumWithTuples::AA:8',
                          'main::EnumWithTuples::BB:8',
                          'main::EnumWithTuples::BC:8',
                          'main::EnumWithTuples::CC:8'])

    def test_enum_with_tuples_values_are_correct_a(self):
        # static ENUM_WITH_TUPLES_A: EnumWithTuples = EnumWithTuples::A(13);
        self.assertEqual(
            self.getFromGlobal("ENUM_WITH_TUPLES_A").getCurrentValue().GetChildAtIndex(0).GetData().GetUnsignedInt8(
                lldb.SBError(), 0),
            13)

    def test_enum_with_tuples_values_are_correct_aa(self):
        # static ENUM_WITH_TUPLES_AA: EnumWithTuples = EnumWithTuples::AA(13, 37);
        value = self.getFromGlobal("ENUM_WITH_TUPLES_AA").getCurrentValue()
        self.assertEqual(
            (value.GetChildAtIndex(0).GetData().GetUnsignedInt8(
                lldb.SBError(), 0),
             value.GetChildAtIndex(1).GetData().GetUnsignedInt8(
                 lldb.SBError(), 0)),
            (13, 37))

    def test_enum_with_tuples_values_are_correct_b(self):
        # static ENUM_WITH_TUPLES_B: EnumWithTuples = EnumWithTuples::B(37);
        self.assertEqual(
            self.getFromGlobal("ENUM_WITH_TUPLES_B").getCurrentValue().GetChildAtIndex(0).GetData().GetUnsignedInt16(
                lldb.SBError(), 0),
            37)

    def test_enum_with_tuples_values_are_correct_bb(self):
        # static ENUM_WITH_TUPLES_BB: EnumWithTuples = EnumWithTuples::BB(37, 5535);
        value = self.getFromGlobal("ENUM_WITH_TUPLES_BB").getCurrentValue()
        self.assertEqual(
            (value.GetChildAtIndex(0).GetData().GetUnsignedInt16(lldb.SBError(), 0),
             value.GetChildAtIndex(1).GetData().GetUnsignedInt16(lldb.SBError(), 0)),
            (37, 5535))

    def test_enum_with_tuples_values_are_correct_bc(self):
        # static ENUM_WITH_TUPLES_BC: EnumWithTuples = EnumWithTuples::BC(65000, 165000);
        value = self.getFromGlobal("ENUM_WITH_TUPLES_BC").getCurrentValue()
        self.assertEqual(
            (value.GetChildAtIndex(0).GetData().GetUnsignedInt16(lldb.SBError(), 0),
             value.GetChildAtIndex(1).GetData().GetUnsignedInt32(lldb.SBError(), 0)),
            (65000, 165000))

    def test_enum_with_tuples_values_are_correct_c(self):
        # static ENUM_WITH_TUPLES_C: EnumWithTuples = EnumWithTuples::C(31337);
        self.assertEqual(
            self.getFromGlobal("ENUM_WITH_TUPLES_C").getCurrentValue().GetChildAtIndex(0).GetData().GetUnsignedInt32(
                lldb.SBError(), 0),
            31337)

    def test_enum_with_tuples_values_are_correct_cc(self):
        # static ENUM_WITH_TUPLES_CC: EnumWithTuples = EnumWithTuples::CC(31337, 87236);
        value = self.getFromGlobal("ENUM_WITH_TUPLES_CC").getCurrentValue()
        self.assertEqual(
            (value.GetChildAtIndex(0).GetData().GetUnsignedInt32(lldb.SBError(), 0),
             value.GetChildAtIndex(1).GetData().GetUnsignedInt32(lldb.SBError(), 0)),
            (31337, 87236))

    def test_enum_with_tuples_values_are_correct_d(self):
        # static ENUM_WITH_TUPLES_D: EnumWithTuples = EnumWithTuples::D(123456789012345678);
        self.assertEqual(
            self.getFromGlobal("ENUM_WITH_TUPLES_D").getCurrentValue().GetChildAtIndex(0).GetData().GetUnsignedInt64(
                lldb.SBError(), 0),
            123456789012345678)

    def test_mixed_enum_variants(self):
        # static MIXED_ENUM_A: MixedEnum1 = MixedEnum1::A;
        self.assertEqual(self.getFromGlobal("MIXED_ENUM_A").getAllVariantTypes(), ['main::MixedEnum::A:64',
                                                                                   'main::MixedEnum::B:64',
                                                                                   'main::MixedEnum::C:64',
                                                                                   'main::MixedEnum::D:64',
                                                                                   'main::MixedEnum::E:64'])

    def test_mixed_enum_a(self):
        # static MIXED_ENUM_A: MixedEnum = MixedEnum::A;
        value = self.getFromGlobal("MIXED_ENUM_A").getCurrentValue()
        self.assertEqual(value.GetType().GetDisplayTypeName(), "main::MixedEnum::A")
        self.assertEqual(value.GetValue(), None)

    def test_mixed_enum_c(self):
        # static MIXED_ENUM_C: MixedEnum = MixedEnum::C(254, -254);
        value = self.getFromGlobal("MIXED_ENUM_C").getCurrentValue()
        self.assertEqual(
            (value.GetChildAtIndex(0).GetData().GetUnsignedInt8(lldb.SBError(), 0),
             value.GetChildAtIndex(1).GetData().GetSignedInt32(lldb.SBError(), 0)),
            (254, -254))

    def test_mixed_enum_d_none(self):
        # static MIXED_ENUM_D_NONE: MixedEnum = MixedEnum::D(None);
        value = RustEnumValue(self.getFromGlobal("MIXED_ENUM_D_NONE").getCurrentValue().GetChildAtIndex(0))
        self.assertEqual(value.getAllVariantTypes(), ["core::option::Option<main::Struct2>::None<main::Struct2>:32",
                                                      "core::option::Option<main::Struct2>::Some<main::Struct2>:32"])
        self.assertEqual(value.getCurrentValue().GetValue(), None)
        self.assertEqual(value.getCurrentValue().GetType().GetDisplayTypeName(),
                         "core::option::Option<main::Struct2>::None<main::Struct2>")

    def test_mixed_enum_d_some(self):
        # static MIXED_ENUM_D_SOME: MixedEnum = MixedEnum::D(Some(Struct2 {
        #     field: 123456,
        #     inner: Struct1 { field: 123 },
        # }));
        variant_with_option = RustEnumValue(
            self.getFromGlobal("MIXED_ENUM_D_SOME").getCurrentValue().GetChildAtIndex(0))

        value_inside_option = variant_with_option.getCurrentValue().GetChildAtIndex(0)
        self.assertEqual(
            value_inside_option.GetChildMemberWithName("field").GetData().GetUnsignedInt32(lldb.SBError(), 0), 123456)

        self.assertEqual(
            value_inside_option.GetChildMemberWithName("inner").GetChildMemberWithName(
                "field").GetData().GetSignedInt32(lldb.SBError(), 0),
            123)
        self.assertEqual(value_inside_option.GetType().GetDisplayTypeName(), "main::Struct2")

    def test_option_non_null_some_pointer(self):
        type = self.target().FindFirstType("core::option::Option<core::ptr::non_null::NonNull<u64>>")
        # this type is "optimized" by rust compiler so the discriminant isn't present on Some variant of option
        data = [1337]
        pointer_size = self.target().GetAddressByteSize()
        byte_order = self.target().GetByteOrder()
        value = RustEnumValue(self.target().CreateValueFromData("adhoc_value",
                                                                lldb.SBData.CreateDataFromUInt64Array(byte_order,
                                                                                                      pointer_size,
                                                                                                      data),
                                                                type))
        self.assertEqual(value.getFields(), ["$variant$0", "$variant$"])
        self.assertEqual(
            value.getCurrentValue().GetChildAtIndex(0).GetChildMemberWithName("pointer").GetValueAsUnsigned(), 1337)

    def test_option_non_null_none(self):
        type = self.target().FindFirstType("core::option::Option<core::ptr::non_null::NonNull<u64>>")
        # this type is "optimized" by rust compiler so the discriminant isn't present on Some variant of option
        # in this test case 0 is used to represent 'None'
        data = [0]
        pointer_size = self.target().GetAddressByteSize()
        byte_order = self.target().GetByteOrder()
        value = RustEnumValue(self.target().CreateValueFromData("adhoc_value",
                                                                lldb.SBData.CreateDataFromUInt64Array(byte_order,
                                                                                                      pointer_size,
                                                                                                      data),
                                                                type))
        self.assertEqual(value.getFields(), ["$variant$0", "$variant$"])
        self.assertEqual(value.getCurrentValue().GetValue(), None)
        self.assertEqual(value.getCurrentValue().GetType().GetDisplayTypeName(),
                         "core::option::Option<core::ptr::non_null::NonNull<u64>>::None<core::ptr::non_null::NonNull<unsigned long> >")

    def test_niche_layout_with_fields_2(self):
        # static NICHE_W_FIELDS_2_A: NicheLayoutWithFields2 =
        #           NicheLayoutWithFields2::A(NonZeroU32::new(800).unwrap(), 900);
        value = self.getFromGlobal("NICHE_W_FIELDS_2_A").getCurrentValue()
        self.assertEqual(
            (
                value.GetChildAtIndex(0).GetChildAtIndex(0).GetData().GetUnsignedInt32(lldb.SBError(), 0),
                value.GetChildAtIndex(1).GetData().GetUnsignedInt32(lldb.SBError(), 0)
            ),
            (800, 900)
        )

    def test_niche_layout_with_fields_3_a(self):
        # static NICHE_W_FIELDS_3_A: NicheLayoutWithFields3 = NicheLayoutWithFields3::A(137, true);
        value = self.getFromGlobal("NICHE_W_FIELDS_3_A").getCurrentValue()
        self.assertEqual(
            (
                value.GetChildAtIndex(0).GetData().GetUnsignedInt8(lldb.SBError(), 0),
                value.GetChildAtIndex(1).GetData().GetUnsignedInt8(lldb.SBError(), 0),
            ),
            (137, 1)
        )

    def test_niche_layout_with_fields_3_a(self):
        # static NICHE_W_FIELDS_3_C: NicheLayoutWithFields3 = NicheLayoutWithFields3::C(false);
        value = self.getFromGlobal("NICHE_W_FIELDS_3_C").getCurrentValue()
        self.assertEqual(
            value.GetChildAtIndex(0).GetData().GetUnsignedInt8(lldb.SBError(), 0),
            0
        )



"""Test that stepping in/out of trampolines works as expected.
"""



from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestTrampoline(TestBase):
    def setup(self, bkpt_str):
        self.build()
        
        _, _, thread, _ = lldbutil.run_to_source_breakpoint(
            self, bkpt_str, lldb.SBFileSpec('main.c'))
        return thread

    def test_direct_call(self):
        thread = self.setup('Break here for direct')

        # Sanity check that we start out in the correct function.
        name = thread.frames[0].GetFunctionName()
        self.assertIn('direct_trampoline_call', name)

        # Check that stepping in will take us directly to the trampoline target.
        thread.StepInto()
        name = thread.frames[0].GetFunctionName()
        self.assertIn('foo', name)

        # Check that stepping out takes us back to the trampoline caller.
        thread.StepOut()
        name = thread.frames[0].GetFunctionName()
        self.assertIn('direct_trampoline_call', name)

        # Check that stepping over the end of the trampoline target 
        # takes us back to the trampoline caller.
        thread.StepInto()
        thread.StepOver()
        name = thread.frames[0].GetFunctionName()
        self.assertIn('direct_trampoline_call', name)


    def test_chained_call(self):
        thread = self.setup('Break here for chained')

        # Sanity check that we start out in the correct function.
        name = thread.frames[0].GetFunctionName()
        self.assertIn('chained_trampoline_call', name)

        # Check that stepping in will take us directly to the trampoline target.
        thread.StepInto()
        name = thread.frames[0].GetFunctionName()
        self.assertIn('foo', name)

        # Check that stepping out takes us back to the trampoline caller.
        thread.StepOut()
        name = thread.frames[0].GetFunctionName()
        self.assertIn('chained_trampoline_call', name)

        # Check that stepping over the end of the trampoline target 
        # takes us back to the trampoline caller.
        thread.StepInto()
        thread.StepOver()
        name = thread.frames[0].GetFunctionName()
        self.assertIn('chained_trampoline_call', name)

    def test_trampoline_after_nodebug(self):
        thread = self.setup('Break here for nodebug then trampoline')

        # Sanity check that we start out in the correct function.
        name = thread.frames[0].GetFunctionName()
        self.assertIn('trampoline_after_nodebug', name)

        # Check that stepping in will take us directly to the trampoline target.
        thread.StepInto()
        name = thread.frames[0].GetFunctionName()
        self.assertIn('foo', name)

        # Check that stepping out takes us back to the trampoline caller.
        thread.StepOut()
        name = thread.frames[0].GetFunctionName()
        self.assertIn('trampoline_after_nodebug', name)

        # Check that stepping over the end of the trampoline target 
        # takes us back to the trampoline caller.
        thread.StepInto()
        thread.StepOver()
        name = thread.frames[0].GetFunctionName()
        self.assertIn('trampoline_after_nodebug', name)

    def test_unused_target(self):
        thread = self.setup('Break here for unused')

        # Sanity check that we start out in the correct function.
        name = thread.frames[0].GetFunctionName()
        self.assertIn('unused_target', name)

        # Check that stepping into a trampoline that doesn't call its target
        # jumps back to its caller.
        thread.StepInto()
        name = thread.frames[0].GetFunctionName()
        self.assertIn('unused_target', name)
        



#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

try:
    from ..ir import *
    from ._ods_common import get_op_result_or_value as _get_op_result_or_value
    from ..dialects import pdl, transform
except ImportError as e:
    raise RuntimeError("Error loading imports from extension module") from e

from typing import List, Optional, Sequence, Tuple, Union, overload

IntOrAttrList = Sequence[Union[IntegerAttr, int]]
OptionalIntList = Optional[Union[ArrayAttr, IntOrAttrList]]

BoolOrAttrList = Sequence[Union[BoolAttr, bool]]
OptionalBoolList = Optional[Union[ArrayAttr, BoolOrAttrList]]

MixedValues = Union[
    Sequence[Union[int, IntegerAttr, Operation, Value, OpView]],
    ArrayAttr,
    Operation,
    Value,
    OpView,
]


# Dispatches `MixedValues` that all represents integers in various forms into
# the following three categories:
#   - `dynamic_values`: a list of `Value`s, potentially from op results;
#   - `packed_values`: a value handle, potentially from an op result, associated
#                      to one or more payload operations of integer type;
#   - `static_values`: an `ArrayAttr` of `i64`s with static values, from Python
#                      `int`s, from `IntegerAttr`s, or from an `ArrayAttr`.
# The input is in the form for `packed_values`, only that result is set and the
# other two are empty. Otherwise, the input can be a mix of the other two forms,
# and for each dynamic value, a special value is added to the `static_values`.
def _dispatch_mixed_values(
    values: MixedValues,
) -> Tuple[List[Value], Union[Operation, Value, OpView], DenseI64ArrayAttr]:
    dynamic_values = []
    packed_values = None
    static_values = None
    if isinstance(values, ArrayAttr):
        static_values = values
    elif isinstance(values, (Operation, Value, OpView)):
        packed_values = values
    else:
        static_values = []
        for size in values or []:
            if isinstance(size, int):
                static_values.append(size)
            else:
                static_values.append(ShapedType.get_dynamic_size())
                dynamic_values.append(_get_op_result_or_value(size))
        static_values = DenseI64ArrayAttr.get(static_values)

    return (dynamic_values, packed_values, static_values)


def _get_int_int_array_attr(
    values: Optional[Union[ArrayAttr, Sequence[Union[ArrayAttr, IntOrAttrList]]]]
) -> ArrayAttr:
    """Creates an array attribute containing array attributes of integers.

    If the operand is already an array attribute, forwards it. Otherwise treats
    the operand as a list of attributes or integers, potentially interpserced, to
    create a new array-of-array attribute. Expects the thread-local MLIR context
    to have been set by the context manager.
    """
    if values is None:
        return ArrayAttr.get([])
    if isinstance(values, ArrayAttr):
        return values
    if isinstance(values, list):
        values = [
            ArrayAttr.get(
                [IntegerAttr.get(IntegerType.get_signless(64), v) for v in value]
            )
            for value in values
        ]

    return ArrayAttr.get(values)


class DecomposeOp:
    """Specialization for DecomposeOp class."""

    def __init__(self, target: Union[Operation, Value], *, loc=None, ip=None):
        super().__init__(
            pdl.OperationType.get(), _get_op_result_or_value(target), loc=loc, ip=ip
        )


class FuseIntoContainingOp:
    """Specialization for FuseIntoContainingOp class."""

    @overload
    def __init__(
        self,
        fused_op_type: Type,
        new_containing_op_type: Type,
        producer_op: Union[Operation, OpView, Value],
        containing_op: Union[Operation, OpView, Value],
        *,
        loc=None,
        ip=None,
    ):
        ...

    @overload
    def __init__(
        self,
        producer_op: Union[Operation, OpView, Value],
        containing_op: Union[Operation, OpView, Value],
        *,
        loc=None,
        ip=None,
    ):
        ...

    def __init__(
        self,
        fused_op_type_or_producer_op: Union[Operation, OpView, Type, Value],
        new_containing_op_type_or_containing_op: Union[Operation, OpView, Type, Value],
        producer_op_or_none: Optional[Union[Operation, OpView, Value]] = None,
        containing_op_or_none: Optional[Union[Operation, OpView, Value]] = None,
        *,
        loc=None,
        ip=None,
    ):
        if isinstance(fused_op_type_or_producer_op, Type):
            if not isinstance(new_containing_op_type_or_containing_op, Type):
                raise TypeError(
                    "If 'fused_op_type_or_producer_op' is a type, then "
                    "'new_containing_op_type_or_containing_op' is expected "
                    "to be one as well."
                )
            fused_op_type = fused_op_type_or_producer_op
            new_containing_op_type = new_containing_op_type_or_containing_op
            producer_op = producer_op_or_none
            containing_op = containing_op_or_none
        else:
            fused_op_type = transform.AnyOpType.get()
            new_containing_op_type = transform.AnyOpType.get()
            producer_op = fused_op_type_or_producer_op
            containing_op = new_containing_op_type_or_containing_op

        super().__init__(
            fused_op_type,
            new_containing_op_type,
            producer_op,
            containing_op,
            loc=loc,
            ip=ip,
        )


class GeneralizeOp:
    """Specialization for GeneralizeOp class."""

    def __init__(self, target: Union[Operation, Value], *, loc=None, ip=None):
        super().__init__(
            pdl.OperationType.get(), _get_op_result_or_value(target), loc=loc, ip=ip
        )


class InterchangeOp:
    """Specialization for InterchangeOp class."""

    def __init__(
        self,
        target: Union[Operation, Value],
        *,
        iterator_interchange: OptionalIntList = None,
        loc=None,
        ip=None,
    ):
        pdl_operation_type = pdl.OperationType.get()
        super().__init__(
            pdl_operation_type,
            _get_op_result_or_value(target),
            iterator_interchange=iterator_interchange,
            loc=loc,
            ip=ip,
        )


class MatchOp:
    """Specialization for MatchOp class."""

    @overload
    @classmethod
    def match_op_names(
        cls,
        target: Union[Operation, Value],
        names: Union[str, Sequence[str]],
        *,
        loc=None,
        ip=None,
    ):
       ...

    @overload
    @classmethod
    def match_op_names(
        cls,
        result_type: Type,
        target: Union[Operation, Value],
        names: Union[str, Sequence[str]],
        *,
        loc=None,
        ip=None,
    ):
       ...

    @classmethod
    def match_op_names(
        cls,
        result_type_or_target: Union[Type, Operation, Value],
        target_or_names: Union[Operation, Value, Sequence[str], str],
        names_or_none: Optional[Union[Sequence[str], str]] = None,
        *,
        loc=None,
        ip=None,
    ):
        if isinstance(result_type_or_target, Type):
           result_type = result_type_or_target
           target = target_or_names
           names = names_or_none
        else:
           result_type = transform.AnyOpType.get()
           target = result_type_or_target
           names = target_or_names

        if isinstance(names, str):
           names = [names]

        return cls(
            result_type,
            _get_op_result_or_value(target),
            ops=ArrayAttr.get(list(map(lambda s: StringAttr.get(s), names))),
            loc=loc,
            ip=ip,
        )


class MultiTileSizesOp:
    """Specialization for MultitileSizesOp class."""

    def __init__(
        self,
        result_type: Type,
        target: Union[Operation, Value],
        *,
        dimension: Union[int, IntegerAttr],
        target_size: Union[int, IntegerAttr],
        divisor: Optional[Optional[Union[int, IntegerAttr]]] = None,
        loc=None,
        ip=None,
    ):
        if divisor is None:
            divisor = 1
        super().__init__(
            result_type,
            result_type,
            result_type,
            _get_op_result_or_value(target),
            dimension=dimension,
            target_size=target_size,
            divisor=divisor,
            loc=loc,
            ip=ip,
        )


class PadOp:
  """Specialization for PadOp class."""

  def __init__(
      self,
      target: Union[Operation, Value],
      *,
      padding_values: Optional[
          Optional[Union[ArrayAttr, Sequence[Attribute]]]
      ] = None,
      padding_dimensions: OptionalIntList = None,
      pack_paddings: OptionalIntList = None,
      transpose_paddings: Optional[
          Union[ArrayAttr, Sequence[Union[ArrayAttr, IntOrAttrList]]]
      ] = None,
      loc=None,
      ip=None,
  ):
    if transpose_paddings is None:
      transpose_paddings = []
    if pack_paddings is None:
      pack_paddings = []
    if padding_dimensions is None:
      padding_dimensions = []
    if padding_values is None:
      padding_values = []
    pdl_operation_type = pdl.OperationType.get()
    transpose_paddings_attr = _get_int_int_array_attr(transpose_paddings)
    super().__init__(
        pdl_operation_type,
        pdl_operation_type,
        _get_op_result_or_value(target),
        padding_values=padding_values,
        padding_dimensions=padding_dimensions,
        pack_paddings=pack_paddings,
        transpose_paddings=transpose_paddings_attr,
        loc=loc,
        ip=ip,
    )


class ScalarizeOp:
    """Specialization for ScalarizeOp class."""

    def __init__(self, target: Union[Operation, Value], *, loc=None, ip=None):
        pdl_operation_type = pdl.OperationType.get()
        super().__init__(
            pdl_operation_type, _get_op_result_or_value(target), loc=loc, ip=ip
        )


class SplitOp:
    """Specialization for SplitOp class."""

    def __init__(
        self,
        target: Union[Operation, Value],
        dimension: Union[int, Attribute],
        split_point: Union[int, Operation, Value, Attribute],
        *,
        loc=None,
        ip=None,
    ):
        if isinstance(split_point, int):
            static_split_point = split_point
            dynamic_split_point = None
        else:
            static_split_point = ShapedType.get_dynamic_size()
            dynamic_split_point = _get_op_result_or_value(split_point)

        target = _get_op_result_or_value(target)

        super().__init__(
            target.type,
            target.type,
            target,
            dimension=dimension,
            static_split_point=static_split_point,
            dynamic_split_point=dynamic_split_point,
            loc=loc,
            ip=ip,
        )


class TileOp:
  """Specialization for TileOp class."""

  @overload
  def __init__(
        self,
        loop_types: Union[Type, List[Type]],
        target: Union[Operation, Value],
        *,
        sizes: Optional[
            Union[Sequence[Union[int, IntegerAttr, Operation, Value]], ArrayAttr]
        ] = None,
        interchange: OptionalIntList = None,
        scalable_sizes: OptionalBoolList = None,
        loc=None,
        ip=None,
    ):
    ...

  @overload
  def __init__(
        self,
        target: Union[Operation, Value, OpView],
        *,
        sizes: Optional[
            Union[Sequence[Union[int, IntegerAttr, Operation, Value]], ArrayAttr]
        ] = None,
        interchange: OptionalIntList = None,
        scalable_sizes: OptionalBoolList = None,
        loc=None,
        ip=None,
    ):
    ...

  def __init__(
        self,
        loop_types_or_target: Union[Type, List[Type], Operation, Value],
        target_or_none: Optional[Union[Operation, Value, OpView]] = None,
        *,
        sizes: Optional[
            Union[Sequence[Union[int, IntegerAttr, Operation, Value]], ArrayAttr]
        ] = None,
        interchange: OptionalIntList = None,
        scalable_sizes: OptionalBoolList = None,
        loc=None,
        ip=None,
    ):
    if interchange is None:
      interchange = []
    if sizes is None:
      sizes = []

    static_sizes = []
    dynamic_sizes = []
    if isinstance(sizes, ArrayAttr):
      sizes_attr = sizes
    else:
      for size in sizes:
        if isinstance(size, int):
          static_sizes.append(size)
        else:
          static_sizes.append(ShapedType.get_dynamic_size())
          dynamic_sizes.append(_get_op_result_or_value(size))
      sizes_attr = DenseI64ArrayAttr.get(static_sizes)

    num_loops = sum(
        v if v == 0 else 1 for v in self.__extract_values(sizes_attr)
    )
    if scalable_sizes is None:
      scalable_sizes = [False] * len(self.__extract_values(sizes_attr))

    if isinstance(loop_types_or_target, (Operation, Value, OpView)):
      loop_types = [transform.AnyOpType.get()] * num_loops
      target = loop_types_or_target
      assert target_or_none is None, "Cannot construct TileOp with two targets."
    else:
      loop_types = (
          ([loop_types_or_target] * num_loops)
          if isinstance(loop_types_or_target, Type)
          else loop_types_or_target
      )
      target = target_or_none

    target = _get_op_result_or_value(target)

    super().__init__(
            target.type,
            loop_types,
            target,
            dynamic_sizes=dynamic_sizes,
            static_sizes=sizes_attr,
            interchange=interchange,
            scalable_sizes=scalable_sizes,
            loc=loc,
            ip=ip,
        )

  def __extract_values(self, attr: Optional[DenseI64ArrayAttr]) -> List[int]:
    if not attr:
      return []
    return [element for element in attr]


class TileToForallOp:
    """Specialization for TileToForallOp class."""

    @overload
    def __init__(
        self,
        loops_type: Type,
        tiled_op_type: Type,
        target: Union[Operation, Value, OpView],
        *,
        num_threads: Optional[MixedValues] = None,
        tile_sizes: MixedValues = None,
        mapping=None,
        loc=None,
        ip=None,
    ):
        ...

    @overload
    def __init__(
        self,
        target: Union[Operation, Value, OpView],
        *,
        num_threads: Optional[MixedValues] = None,
        tile_sizes: MixedValues = None,
        mapping=None,
        loc=None,
        ip=None,
    ):
        ...

    def __init__(
        self,
        loops_type_or_target: Union[
            Type, Union[Operation, Value, OpView]  # loops_type
        ],  # target
        tiled_op_type_or_none: Optional[Type] = None,
        target_or_none: Optional[Union[Operation, Value, OpView]] = None,
        *,
        num_threads: MixedValues = None,
        tile_sizes: MixedValues = None,
        mapping=None,
        loc=None,
        ip=None,
    ):
        # `Type` arguments in the front are optional: add default values to front.
        if isinstance(loops_type_or_target, Type):
            # First overload: type arguments provided.
            if not isinstance(tiled_op_type_or_none, Type):
                raise TypeError(
                    "If 'loops_type_or_target' is a type, then "
                    "'tiled_op_type_or_none' is expected to be one as well."
                )
            loops_type = loops_type_or_target
            tiled_op_type = tiled_op_type_or_none
            target = target_or_none
        else:
            # Last overload: type arguments missing.
            loops_type = transform.AnyOpType.get()
            tiled_op_type = transform.AnyOpType.get()
            target = loops_type_or_target

        # Unpack mixed num_threads.
        (
            dynamic_num_threads,
            packed_num_threads,
            num_threads_attr,
        ) = _dispatch_mixed_values(num_threads)

        # Unpack mixed tile_sizes.
        (
            dynamic_tile_sizes,
            packed_tile_sizes,
            tile_sizes_attr,
        ) = _dispatch_mixed_values(tile_sizes)

        super().__init__(
            loops_type,
            tiled_op_type,
            target=target,
            tile_sizes=dynamic_tile_sizes,
            packed_tile_sizes=packed_tile_sizes,
            static_tile_sizes=tile_sizes_attr,
            num_threads=dynamic_num_threads,
            packed_num_threads=packed_num_threads,
            static_num_threads=num_threads_attr,
            mapping=mapping,
            loc=loc,
            ip=ip,
        )


class VectorizeOp:
    """Specialization for VectorizeOp class."""

    def __init__(
        self,
        target: Union[Operation, Value],
        *,
        vectorize_padding: Union[bool, BoolAttr] = False,
        loc=None,
        ip=None,
    ):
        pdl_operation_type = pdl.OperationType.get()
        if isinstance(vectorize_padding, bool):
            vectorize_padding = UnitAttr.get()
        super().__init__(
            pdl_operation_type,
            _get_op_result_or_value(target),
            vectorize_padding=vectorize_padding,
            loc=loc,
            ip=ip,
        )



"""Test that we can unwind out of a signal handler.
   Which for AArch64 Linux requires a specific unwind plan."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class UnwindSignalTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_unwind_signal(self):
        """Inferior calls sigill() and handles the resultant SIGILL.
        Stopped at a breakpoint in the handler, check that we can unwind
        back to sigill() and get the expected register contents there."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertState(process.GetState(), lldb.eStateStopped)
        signo = process.GetUnixSignals().GetSignalNumberFromName("SIGILL")

        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonSignal)
        self.assertTrue(
            thread and thread.IsValid(), "Thread should be stopped due to a signal"
        )
        self.assertTrue(
            thread.GetStopReasonDataCount() >= 1, "There should be data in the event."
        )
        self.assertEqual(
            thread.GetStopReasonDataAtIndex(0),
            signo,
            "The stop signal should be SIGILL",
        )

        # Continue to breakpoint in sigill handler
        bkpt = target.FindBreakpointByID(
            lldbutil.run_break_set_by_source_regexp(self, "Set a breakpoint here")
        )
        threads = lldbutil.continue_to_breakpoint(process, bkpt)
        self.assertEqual(len(threads), 1, "Expected single thread")
        thread = threads[0]

        # Expect breakpoint in 'handler'
        frame = thread.GetFrameAtIndex(0)
        self.assertEqual(frame.GetDisplayFunctionName(), "handler", "Unexpected break?")

        # Expect that unwinding should find 'sigill'
        found_caller = False
        for frame in thread.get_thread_frames():
            if frame.GetDisplayFunctionName() == "sigill":
                # We should have ascending values in the x registers
                regs = frame.GetRegisters().GetValueAtIndex(0)
                err = lldb.SBError()

                for i in range(31):
                    name = "x{}".format(i)
                    value = regs.GetChildMemberWithName(name).GetValueAsUnsigned(err)
                    self.assertSuccess(err, "Failed to get register {}".format(name))
                    self.assertEqual(
                        value, i, "Unexpected value for register {}".format(name)
                    )

                found_caller = True
                break

        self.assertTrue(
            found_caller, "Unwinding did not find func that caused the SIGILL"
        )

        # Continue until we exit.
        process.Continue()
        self.assertState(process.GetState(), lldb.eStateExited)
        self.assertEqual(process.GetExitStatus(), 0)



"""Test that types defined in shared libraries with stripped symbols work correctly."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SharedLibStrippedTestCase(TestBase):
    @expectedFailureAll(oslist=["windows"])
    def test_expr(self):
        """Test that types work when defined in a shared library and forwa/d-declared in the main executable"""
        if "clang" in self.getCompiler() and "3.4" in self.getCompilerVersion():
            self.skipTest(
                "llvm.org/pr16214 -- clang emits partial DWARF for structures referenced via typedef"
            )

        self.build()
        self.common_setup()

        # This should display correctly.
        self.expect(
            "expression --show-types -- *my_foo_ptr",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["(foo)", "(sub_foo)", "other_element = 3"],
        )

    @expectedFailureAll(oslist=["windows"])
    @expectedFailure("llvm.org/PR36712")
    def test_frame_variable(self):
        """Test that types work when defined in a shared library and forward-declared in the main executable"""
        self.build()
        self.common_setup()

        # This should display correctly.
        self.expect(
            "frame variable --show-types -- *my_foo_ptr",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["(foo)", "(sub_foo)", "other_element = 3"],
        )

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.source = "main.c"
        self.line = line_number(self.source, "// Set breakpoint 0 here.")
        self.shlib_names = ["foo"]

    def common_setup(self):
        # Run in synchronous mode
        self.dbg.SetAsync(False)

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target, VALID_TARGET)

        # Break inside the foo function which takes a bar_ptr argument.
        lldbutil.run_break_set_by_file_and_line(
            self, self.source, self.line, num_expected_locations=1, loc_exact=True
        )

        # Register our shared libraries for remote targets so they get
        # automatically uploaded
        environment = self.registerSharedLibrariesWithTarget(target, self.shlib_names)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(
            None, environment, self.get_process_working_directory()
        )
        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)



"""Test stepping over vrs. hitting breakpoints & subsequent stepping in various forms."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCStepping(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers that we will step to in main:
        self.main_source = "main.c"

    @add_test_categories(["pyapi", "basic_process"])
    @expectedFailureAll(oslist=["freebsd"], bugnumber="llvm.org/pr17932")
    @expectedFailureAll(oslist=["linux"], archs=no_match(["i386", "x86_64"]))
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24777")
    @expectedFailureNetBSD
    def test_and_python_api(self):
        """Test stepping over vrs. hitting breakpoints & subsequent stepping in various forms."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        self.main_source_spec = lldb.SBFileSpec(self.main_source)

        breakpoints_to_disable = []

        break_1_in_main = target.BreakpointCreateBySourceRegex(
            "// frame select 2, thread step-out while stopped at .c.1..",
            self.main_source_spec,
        )
        self.assertTrue(break_1_in_main, VALID_BREAKPOINT)
        breakpoints_to_disable.append(break_1_in_main)

        break_in_a = target.BreakpointCreateBySourceRegex(
            "// break here to stop in a before calling b", self.main_source_spec
        )
        self.assertTrue(break_in_a, VALID_BREAKPOINT)
        breakpoints_to_disable.append(break_in_a)

        break_in_b = target.BreakpointCreateBySourceRegex(
            "// thread step-out while stopped at .c.2..", self.main_source_spec
        )
        self.assertTrue(break_in_b, VALID_BREAKPOINT)
        breakpoints_to_disable.append(break_in_b)

        break_in_c = target.BreakpointCreateBySourceRegex(
            "// Find the line number of function .c. here.", self.main_source_spec
        )
        self.assertTrue(break_in_c, VALID_BREAKPOINT)
        breakpoints_to_disable.append(break_in_c)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, break_1_in_main)

        if len(threads) != 1:
            self.fail("Failed to stop at first breakpoint in main.")

        thread = threads[0]

        # Get the stop id and for fun make sure it increases:
        old_stop_id = process.GetStopID()

        # Now step over, which should cause us to hit the breakpoint in "a"
        thread.StepOver()

        # The stop reason of the thread should be breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, break_in_a)
        if len(threads) != 1:
            self.fail("Failed to stop at breakpoint in a.")

        # Check that the stop ID increases:
        new_stop_id = process.GetStopID()
        self.assertTrue(new_stop_id > old_stop_id, "Stop ID increases monotonically.")

        thread = threads[0]

        # Step over, and we should hit the breakpoint in b:
        thread.StepOver()

        threads = lldbutil.get_threads_stopped_at_breakpoint(process, break_in_b)
        if len(threads) != 1:
            self.fail("Failed to stop at breakpoint in b.")
        thread = threads[0]

        # Now try running some function, and make sure that we still end up in the same place
        # and with the same stop reason.
        frame = thread.GetFrameAtIndex(0)
        current_line = frame.GetLineEntry().GetLine()
        current_file = frame.GetLineEntry().GetFileSpec()
        current_bp = []
        current_bp.append(thread.GetStopReasonDataAtIndex(0))
        current_bp.append(thread.GetStopReasonDataAtIndex(1))

        stop_id_before_expression = process.GetStopID()
        stop_id_before_including_expressions = process.GetStopID(True)

        frame.EvaluateExpression("(int) printf (print_string)")

        frame = thread.GetFrameAtIndex(0)
        self.assertEqual(
            current_line,
            frame.GetLineEntry().GetLine(),
            "The line stayed the same after expression.",
        )
        self.assertEqual(
            current_file,
            frame.GetLineEntry().GetFileSpec(),
            "The file stayed the same after expression.",
        )
        self.assertEqual(
            thread.GetStopReason(),
            lldb.eStopReasonBreakpoint,
            "We still say we stopped for a breakpoint.",
        )
        self.assertTrue(
            thread.GetStopReasonDataAtIndex(0) == current_bp[0]
            and thread.GetStopReasonDataAtIndex(1) == current_bp[1],
            "And it is the same breakpoint.",
        )

        # Also make sure running the expression didn't change the public stop id
        # but did change if we are asking for expression stops as well.
        stop_id_after_expression = process.GetStopID()
        stop_id_after_including_expressions = process.GetStopID(True)

        self.assertEqual(
            stop_id_before_expression,
            stop_id_after_expression,
            "Expression calling doesn't change stop ID",
        )

        self.assertTrue(
            stop_id_after_including_expressions > stop_id_before_including_expressions,
            "Stop ID including expressions increments over expression call.",
        )

        # Do the same thing with an expression that's going to crash, and make
        # sure we are still unchanged.

        frame.EvaluateExpression("((char *) 0)[0] = 'a'")

        frame = thread.GetFrameAtIndex(0)
        self.assertEqual(
            current_line,
            frame.GetLineEntry().GetLine(),
            "The line stayed the same after expression.",
        )
        self.assertEqual(
            current_file,
            frame.GetLineEntry().GetFileSpec(),
            "The file stayed the same after expression.",
        )
        self.assertEqual(
            thread.GetStopReason(),
            lldb.eStopReasonBreakpoint,
            "We still say we stopped for a breakpoint.",
        )
        self.assertTrue(
            thread.GetStopReasonDataAtIndex(0) == current_bp[0]
            and thread.GetStopReasonDataAtIndex(1) == current_bp[1],
            "And it is the same breakpoint.",
        )

        # Now continue and make sure we just complete the step:
        # Disable all our breakpoints first - sometimes the compiler puts two line table entries in for the
        # breakpoint a "b" and we don't want to hit that.
        for bkpt in breakpoints_to_disable:
            bkpt.SetEnabled(False)

        process.Continue()

        self.assertEqual(thread.GetFrameAtIndex(0).GetFunctionName(), "a")
        self.assertStopReason(thread.GetStopReason(), lldb.eStopReasonPlanComplete)

        # And one more time should get us back to main:
        process.Continue()

        self.assertEqual(thread.GetFrameAtIndex(0).GetFunctionName(), "main")
        self.assertStopReason(thread.GetStopReason(), lldb.eStopReasonPlanComplete)

        # Now make sure we can call a function, break in the called function,
        # then have "continue" get us back out again:
        frame = thread.GetFrameAtIndex(0)
        frame = thread.GetFrameAtIndex(0)
        current_line = frame.GetLineEntry().GetLine()
        current_file = frame.GetLineEntry().GetFileSpec()

        break_in_b.SetEnabled(True)
        options = lldb.SBExpressionOptions()
        options.SetIgnoreBreakpoints(False)
        options.SetFetchDynamicValue(False)
        options.SetUnwindOnError(False)
        frame.EvaluateExpression("b (4)", options)

        threads = lldbutil.get_threads_stopped_at_breakpoint(process, break_in_b)

        if len(threads) != 1:
            self.fail("Failed to stop at breakpoint in b when calling b.")
        thread = threads[0]

        # So do a step over here to make sure we can still do that:

        thread.StepOver()

        # See that we are still in b:
        func_name = thread.GetFrameAtIndex(0).GetFunctionName()
        self.assertEqual(func_name, "b", "Should be in 'b', were in %s" % (func_name))

        # Okay, now if we continue, we will finish off our function call and we
        # should end up back in "a" as if nothing had happened:
        process.Continue()

        self.assertEqual(
            thread.GetFrameAtIndex(0).GetLineEntry().GetLine(), current_line
        )
        self.assertEqual(
            thread.GetFrameAtIndex(0).GetLineEntry().GetFileSpec(), current_file
        )

        # Now we are going to test step in targeting a function:

        break_in_b.SetEnabled(False)

        break_before_complex_1 = target.BreakpointCreateBySourceRegex(
            "// Stop here to try step in targeting b.", self.main_source_spec
        )
        self.assertTrue(break_before_complex_1, VALID_BREAKPOINT)

        break_before_complex_2 = target.BreakpointCreateBySourceRegex(
            "// Stop here to try step in targeting complex.", self.main_source_spec
        )
        self.assertTrue(break_before_complex_2, VALID_BREAKPOINT)

        break_before_complex_3 = target.BreakpointCreateBySourceRegex(
            "// Stop here to step targeting b and hitting breakpoint.",
            self.main_source_spec,
        )
        self.assertTrue(break_before_complex_3, VALID_BREAKPOINT)

        break_before_complex_4 = target.BreakpointCreateBySourceRegex(
            "// Stop here to make sure bogus target steps over.", self.main_source_spec
        )
        self.assertTrue(break_before_complex_4, VALID_BREAKPOINT)

        threads = lldbutil.continue_to_breakpoint(process, break_before_complex_1)
        self.assertEqual(len(threads), 1)
        thread = threads[0]
        break_before_complex_1.SetEnabled(False)

        thread.StepInto("b")
        self.assertEqual(thread.GetFrameAtIndex(0).GetFunctionName(), "b")

        # Now continue out and stop at the next call to complex.  This time
        # step all the way into complex:
        threads = lldbutil.continue_to_breakpoint(process, break_before_complex_2)
        self.assertEqual(len(threads), 1)
        thread = threads[0]
        break_before_complex_2.SetEnabled(False)

        thread.StepInto("complex")
        self.assertEqual(thread.GetFrameAtIndex(0).GetFunctionName(), "complex")

        # Now continue out and stop at the next call to complex.  This time
        # enable breakpoints in a and c and then step targeting b:
        threads = lldbutil.continue_to_breakpoint(process, break_before_complex_3)
        self.assertEqual(len(threads), 1)
        thread = threads[0]
        break_before_complex_3.SetEnabled(False)

        break_at_start_of_a = target.BreakpointCreateByName("a")
        break_at_start_of_c = target.BreakpointCreateByName("c")

        thread.StepInto("b")
        threads = lldbutil.get_stopped_threads(process, lldb.eStopReasonBreakpoint)

        self.assertEqual(len(threads), 1)
        thread = threads[0]
        stop_break_id = thread.GetStopReasonDataAtIndex(0)
        self.assertTrue(
            stop_break_id == break_at_start_of_a.GetID()
            or stop_break_id == break_at_start_of_c.GetID()
        )

        break_at_start_of_a.SetEnabled(False)
        break_at_start_of_c.SetEnabled(False)

        process.Continue()
        self.assertEqual(thread.GetFrameAtIndex(0).GetFunctionName(), "b")

        # Now continue out and stop at the next call to complex.  This time
        # enable breakpoints in a and c and then step targeting b:
        threads = lldbutil.continue_to_breakpoint(process, break_before_complex_4)
        self.assertEqual(len(threads), 1)
        thread = threads[0]
        break_before_complex_4.SetEnabled(False)

        thread.StepInto("NoSuchFunction")
        self.assertEqual(thread.GetFrameAtIndex(0).GetFunctionName(), "main")



# TestExprInProtocolExtension.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Tests scoped variables with swift expressions
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftExprInProtocolExtension(TestBase):
    def continue_to_bkpt(self, process, bkpt):
        threads = lldbutil.continue_to_breakpoint(process, bkpt)
        self.assertTrue(len(threads) == 1)

    def continue_by_pattern(self, pattern):
        bkpt = self.target.BreakpointCreateBySourceRegex(
            pattern, self.main_source_spec)
        self.assertTrue(bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)
        self.continue_to_bkpt(self.process, bkpt)
        self.target.BreakpointDelete(bkpt.GetID())

    @swiftTest
    def test_protocol_extension(self):
        """Tests that swift expressions in protocol extension functions behave correctly"""
        self.build()

        # Create the target
        target = self.dbg.CreateTarget(self.getBuildArtifact())
        self.target = target
        self.assertTrue(target, VALID_TARGET)

        # Set the breakpoints
        static_bkpt = target.BreakpointCreateBySourceRegex(
            'break here in static func', lldb.SBFileSpec('main.swift'))
        self.assertTrue(static_bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)

        method_bkpt = target.BreakpointCreateBySourceRegex(
            'break here in method', lldb.SBFileSpec('main.swift'))
        self.assertTrue(method_bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)

        # Launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, os.getcwd())
        self.process = process
        self.assertTrue(process, PROCESS_IS_VALID)

        # Frame #0 should be at our breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(
            process, method_bkpt)

        self.assertTrue(len(threads) == 1)

        # Check that we can evaluate expressions correctly in the struct
        # method.
        lldbutil.check_expression(self, self.frame(), "self.x", "10", False)
        lldbutil.check_expression(self, self.frame(), "self.y", '"Hello world"', True)
        lldbutil.check_expression(self, self.frame(), "local_var", "111", False)

        # And check that we got the type of self right:
        self_var = self.frame().EvaluateExpression(
            "self", lldb.eDynamicCanRunTarget)
        self_type_name = self_var.GetTypeName()
        print("Self type name is: ", self_type_name)

        # Not checking yet since we don't get this right.

        # Now continue to the static method and check things there:
        self.continue_to_bkpt(process, static_bkpt)

        lldbutil.check_expression(self, self.frame(), "self.cvar", "333", False)
        lldbutil.check_expression(self, self.frame(), "local_var", "222", False)

        # This continues to the class version:
        self.continue_to_bkpt(process, method_bkpt)
        # Check that we can evaluate expressions correctly in the struct
        # method.
        lldbutil.check_expression(self, self.frame(), "self.x", "10", False)
        lldbutil.check_expression(self, self.frame(), "self.y", '"Hello world"', True)
        lldbutil.check_expression(self, self.frame(), "local_var", "111", False)

        # And check that we got the type of self right:
        self_var = self.frame().EvaluateExpression(
            "self", lldb.eDynamicCanRunTarget)
        self_type_name = self_var.GetTypeName()
        print("Self type name is: ", self_type_name)

        # Not checking yet since we don't get this right.

        # Now continue to the static method and check things there:
        self.continue_to_bkpt(process, static_bkpt)

        lldbutil.check_expression(self, self.frame(), "self.cvar", "333", False)
        lldbutil.check_expression(self, self.frame(), "local_var", "222", False)

        # This continues to the enum version:
        self.continue_to_bkpt(process, method_bkpt)
        # Check that we can evaluate expressions correctly in the struct
        # method.
        lldbutil.check_expression(self, self.frame(), "self.x", "10", False)
        lldbutil.check_expression(self, self.frame(), "self.y", '"Hello world"', True)
        lldbutil.check_expression(self, self.frame(), "local_var", "111", False)

        # And check that we got the type of self right:
        self_var = self.frame().EvaluateExpression(
            "self", lldb.eDynamicCanRunTarget)
        self_type_name = self_var.GetTypeName()
        print("Self type name is: ", self_type_name)

        # Not checking yet since we don't get this right.

        # Now continue to the static method and check things there:
        self.continue_to_bkpt(process, static_bkpt)

        lldbutil.check_expression(self, self.frame(), "self.cvar", "333", False)
        lldbutil.check_expression(self, self.frame(), "local_var", "222", False)



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class SwiftGenericClassTest(TestBase):
    @swiftTest
    def test(self):
        """Tests that a generic class type can be resolved from the instance metadata alone"""
        self.build()
        (target, process, thread, breakpoint) = lldbutil.run_to_source_breakpoint(self, 
                "break here", lldb.SBFileSpec("main.swift"))

        self.expect("frame variable -d run self",
                    substrs=["a.F<Int>", "23", "42", "128", "256"])
        self.expect("expr -d run -- self",
                    substrs=["a.F<Int>", "23", "42", "128", "256"])



"""
Test the output of `frame diagnose` for calling virtual methods
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestDiagnoseVirtualMethodCall(TestBase):
    @skipUnlessDarwin
    @skipIf(
        archs=no_match(["x86_64"])
    )  # <rdar://problem/33842388> frame diagnose doesn't work for armv7 or arm64
    def test_diagnose_virtual_method_call(self):
        TestBase.setUp(self)
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)
        self.runCmd("run", RUN_SUCCEEDED)
        self.expect("thread list", "Thread should be stopped", substrs=["stopped"])
        self.expect("frame diagnose", "Crash diagnosis was accurate", substrs=["foo"])



"""
This tests LLDB's ability to merge structs into the shared per-target Clang
ASTContext.

This just focuses on indirect imports (i.e., a declaration gets imported from
the lldb::Module AST into the expression AST and then the declaration gets
imported to the scratch AST because it is part of the ValueObject type of the
result) and direct imports (i.e., a declaration gets directly imported from a
lldb::Module AST to the scratch AST, e.g., via 'frame var').
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def common_setup(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

    def do_pass(self, kind, var, expected_type, expected_children):
        if kind == "expression":
            self.expect_expr(
                var, result_type=expected_type, result_children=expected_children
            )
        elif kind == "path":
            self.expect_var_path(var, type=expected_type, children=expected_children)
        else:
            self.fail("Unknown var evaluation kind: " + var)

    def pull_in_and_merge(self, var, type, children):
        """
        Pulls in the specified variable into the scratch AST. Afterwards tries
        merging the declaration. The method of pulling the declaration into the
        scratch AST is defined by the first_pass/second_pass instance variables.
        """

        # This pulls in the declaration into the scratch AST.
        self.do_pass(self.first_pass, var, type, children)
        # This pulls in the declaration a second time and forces us to merge with
        # the existing declaration (or reuse the existing declaration).
        self.do_pass(self.second_pass, var, type, children)

    def do_tests(self):
        """Just forwards all the variables/types/childrens to pull_in_and_merge."""
        self.pull_in_and_merge(
            "decl_in_func", type="DeclInFunc", children=[ValueCheck(name="member")]
        )
        self.pull_in_and_merge(
            "top_level_struct",
            type="TopLevelStruct",
            children=[ValueCheck(name="member")],
        )
        self.pull_in_and_merge(
            "inner_struct",
            type="OuterStruct::InnerStruct",
            children=[ValueCheck(name="member")],
        )
        self.pull_in_and_merge(
            "typedef_struct",
            type="TypedefStruct",
            children=[ValueCheck(name="member")],
        )
        self.pull_in_and_merge(
            "namespace_struct",
            type="NS::NamespaceStruct",
            children=[ValueCheck(name="member")],
        )
        self.pull_in_and_merge(
            "unnamed_namespace_struct",
            type="UnnamedNamespaceStruct",
            children=[ValueCheck(name="member")],
        )
        self.pull_in_and_merge(
            "extern_c_struct",
            type="ExternCStruct",
            children=[ValueCheck(name="member")],
        )

    @no_debug_info_test
    def test_direct_and_indirect(self):
        """
        First variable paths pull in a declaration directly. Then the expression
        evaluator pulls the declaration in indirectly.
        """
        self.common_setup()
        self.first_pass = "path"
        self.second_pass = "expression"
        self.do_tests()

    @no_debug_info_test
    def test_indirect_and_indirect(self):
        """
        The expression evaluator pulls in the declaration indirectly and then
        repeat that process.
        """
        self.common_setup()
        self.first_pass = "expression"
        self.second_pass = "expression"
        self.do_tests()

    @no_debug_info_test
    def test_indirect_and_direct(self):
        """
        The expression evaluator pulls in the declaration indirectly and then
        variable paths pull it in directly.
        """
        self.common_setup()
        self.first_pass = "expression"
        self.second_pass = "path"
        self.do_tests()

    @no_debug_info_test
    def test_direct_and_direct(self):
        """
        Variable paths pulls in the declaration indirectly and then repeat that
        process.
        """
        self.common_setup()
        self.first_pass = "path"
        self.second_pass = "path"
        self.do_tests()
